subroutine apply_lookup(Observation_dimension, Model_dimension, Observation_points_in_radius, &
                                                Model_lookup_table, screened_observations, min_num_obs, &
                                                OBS_on_model, STD_on_model)

!----------------------------------------------------------------------------
!Subroutine to transform observations from observation space to model space.
!Observations are rejected if less than the specified number is available for each pixel.
!Written by DF, 3/3/14 (modified from Matlab code written by Alina Barbu)
!----------------------------------------------------------------------------

implicit none

!Input variables
integer, intent(in) :: Model_dimension   !Dimension of model grid
integer, intent(in) :: Observation_dimension    !Dimension of observation grid
integer, intent(in) :: Observation_points_in_radius   !Max number of model points in radius
integer, intent(in) :: min_num_obs !Minimum number of observations
integer, intent(in) :: Model_lookup_table(0:Model_dimension-1, 0:Observation_points_in_radius-1) !Model lookup table
real, intent(in) :: screened_observations(0:Observation_dimension-1) !Screened observations

!Output variables
real, intent(out) :: OBS_on_model(0:Model_dimension-1) !Observations projected onto model grid
real, intent(out) :: STD_on_model(0:Model_dimension-1) !standard deviation

!Local variables
integer :: i,j,k
real :: temp(0:Observation_points_in_radius-1)

OBS_on_model = 999
STD_on_model = 999

do i=0, Model_dimension-1
  k=0
  temp=0.0
  do j=0, Observation_points_in_radius-1
     if (Model_lookup_table(i,j) .ne. -999) then
       if (screened_observations(Model_lookup_table(i,j)) .ne. 999) then
         temp(j) = screened_observations(Model_lookup_table(i,j)) !Store observations in lookup tables
         k = k+1
       end if
     end if
  end do

  !Transform observations onto model grid:
  if (k .ge. min_num_obs) then
    OBS_on_model(i) = sum(temp(:))/dble(k)
!     STD_on_model(i) = sqrt((1.0/(Observation_points_in_radius-1))*sum(temp(:) - OBS_on_model(i))**2)
  else
    OBS_on_model(i) = 999
  end if
end do

end subroutine apply_lookup
