subroutine Calculate_cdf_parameters(swi, wg, Model_dimension, a, b, Ntimes, N, NSSR)
!----------------------------------------------------------------------------
!Subroutine to calculate the cumulative distribution function parameters for ascat
!soil moisture observations (swi), using model predicted observations (wg). Time series
!Ntimes is used to calculate the parameters.
!Written by DF, 26/2/14 (modified from Matlab code written by )
!----------------------------------------------------------------------------
implicit none

!Input variables
integer, intent(in) :: Model_dimension !Model dimension
integer, intent(in) :: Ntimes !Number of timesteps
real, intent(in) :: wg(0:Model_dimension-1,0:Ntimes-1) !Model predicted soil moisture
real, intent(in) :: swi(0:Model_dimension-1,0:Ntimes-1) !Observed soil moisture

!Output variables
real, intent(out) :: a(0:Model_dimension-1) !Parameter a in cdf matching
real, intent(out) :: b(0:Model_dimension-1) !Parameter b in cdf matching
real, intent(out) :: N(0:Model_dimension-1) !Number of points in cdf matching
real, intent(out) :: NSSR(0:Model_dimension-1) !Normalized residual sum of square

!Local variables
integer :: i,j

real :: square_swi
real :: square_wg
real :: mean_swi !mean swi
real :: mean_wg!mean wg
real :: std_swi !standard deviation of swi
real :: std_wg !standard deviation of wg
real :: sum_residual
real :: sum_tot_var

do i = 0, Model_dimension-1
  N(i) = 0
  NSSR(i) = 0
  Mean_swi = 0.0
  Mean_wg = 0.0
  square_swi = 0.0
  square_wg = 0.0
  do j=0, Ntimes-1
     if (swi(i,j) .ne. 999 .and. wg(i,j) .ne. 999) then
       Mean_swi = Mean_swi + swi(i,j)
       Mean_wg = Mean_wg + wg(i,j)
       square_swi = square_swi + swi(i,j)**2
       square_wg = square_wg + wg(i,j)**2
       N(i)=N(i)+1
     end if
  end do
  if (N(i)>30) then
    !Calculate mean and standard deviation of values
    Mean_swi = Mean_swi/dble(N(i))
    Mean_wg = Mean_wg/dble(N(i))
    Std_swi = sqrt(square_swi/dble(N(i)) - Mean_swi**2)
    Std_wg = sqrt(square_wg/dble(N(i)) - Mean_wg**2)
    !Calculate cdf matching parameters
    if (Std_swi .ne. 0 .and. Std_wg .ne. 0) then
      b(i) = Std_wg/Std_swi
      a(i) = Mean_wg - b(i)*Mean_swi
      !Calculate NSSR
      sum_residual = 0
      sum_tot_var  = 0
      do j=0, Ntimes-1
        if (swi(i,j) .ne. 999 .and. wg(i,j) .ne. 999) then
          sum_residual = sum_residual + (wg(i,j)-(a(i)+b(i)*swi(i,j)))**2
          sum_tot_var  = sum_tot_var  + (wg(i,j)-Mean_wg)**2
        end if
      end do
      NSSR(i) = sqrt(sum_residual/sum_tot_var)
    else
      b(i) = 999
      a(i) = 999
    end if
  else !No valid data to generate parameters
    b(i) = 999
    a(i) = 999
  end if
end do

end subroutine Calculate_cdf_parameters



