subroutine lookup_table_downscale(Model_dimension, Observation_dimension, &
                              Observation_points_in_radius, lati, longi, radius, x, y, &
                              Model_lookup_table, Missing_value)

!----------------------------------------------------------------------------
!Subroutine to generate lookup tables. The model lookup table stores the observations
!within a specified radius of the model points and vice-versa for the observation lookup table
!Written by DF, 26/2/14 (modified from Matlab code written by)
!----------------------------------------------------------------------------

implicit none

!Input variables
integer, intent(in) :: Model_dimension !Model dimension (number of points in model grid)
integer, intent(in) :: Observation_dimension !Observation dimension
integer, intent(in) :: Observation_points_in_radius !Max number of observation points in radius
real, intent(in) :: lati(0:Model_dimension-1) !Latitude coordinates of model
real, intent(in) :: longi(0:Model_dimension-1) !Longitude coordinates of model
real, intent(in) :: x(0:Observation_dimension-1) !latitude coordinates of observations
real, intent(in) :: y(0:Observation_dimension-1) !Longitude coordinates of observations
real, intent(in) :: radius !radius to assimilate gridpoints for lookup tables

!Output variables
integer, intent(out) :: Model_lookup_table(0:Model_dimension-1,0:Observation_points_in_radius-1) !Lookup tables
integer, intent(out) :: Missing_value

!Local variables
integer :: i,j,k
real :: temp
Missing_value = -999
Model_lookup_table = Missing_value !Set values to -999, thereby rejecting points which are not in specified radius

!Generate lookup tables:
do i=0, Model_dimension - 1
  k = 0
  do j=0, Observation_dimension - 1
     temp = (y(j) - lati(i))**2 + (x(j) - longi(i))**2 !Points within specified radius
     if (temp < radius**2) then
       Model_lookup_table(i,k) = j
       k = k+1
     end if
  end do
end do

!Observation_lookup_table = 0
!do i=1, Observation_dimension
!  k = 1
!  do j=1, Model_dimension
!     temp = (y(i) - lati(j))**2 + (x(i) - longi(j))**2
!     if (temp < radius**2) then
!       Observation_lookup_table(i,k) = i
!       k = k+1
!     end if
!  end do
!end do


end subroutine lookup_table_downscale


