subroutine Seasonal_cdf_matching(swi, Model_dimension, a, b, Ntimes)
!----------------------------------------------------------------------------
!Subroutine to perform the cumulative distribution function (cdf) for ascat
!soil moisture observations (swi), using the cdf parameters a and b generated by
!calculate_cdf_parameters.f90. Note that Ntimes should be the time window over
!which the cdf matching parameters a and b are valid.
!Written by DF, 26/2/14 (modified from Matlab code written by )
!----------------------------------------------------------------------------
implicit none

!Input variables
integer, intent(in) :: Model_dimension !Model dimension
integer, intent(in) :: Ntimes !Number of timesteps
real, intent(in) :: a(0:Model_dimension-1) !Parameter a in cdf matching
real, intent(in) :: b(0:Model_dimension-1) !Parameter b in cdf matching

!Output variables
real, intent(inout) :: swi(0:Model_dimension-1,0:Ntimes-1) !Observed soil moisture

!Local variables
integer :: i,j

!Re-scale observed soil moisture over time series using cdf matching
do i=0, Ntimes-1
  do j = 0, Model_dimension-1
    if (a(j) .ne. 999 .and. b(j) .ne. 999 .and. swi(j,i) .ne. 999) then
      swi(j,i) = a(j) + b(j)*swi(j,i)
      if (swi(j,i)<0) swi(j,i) = 0.
    else
      swi(j,i) = 999
    end if
  end do
end do

end subroutine Seasonal_cdf_matching



