
exp_name = 'ekf' # Name of experiment
exp_root = '/cnrm/vegeo/zhengy/LDAS/Global'
# -------------------------------
# ASSIMILATION
# -------------------------------
analysis_period = ['2010-01-01', '2010-12-31'] # Analysis period.
assim_hour = 9 # Assimilation hour (in range(24)).

# -------------------------------
# FORCING
# -------------------------------
get_raw_forcing = True # Retrieve raw forcing files (True) or not (False)
freq_forcing = 1 # Time frequency of the forcing data in hours
nc_version = 'NETCDF4' # NetCDF version; possbile values: 'NETCDF4', 'NETCDF4_CLASSIC', 'NETCDF3_64BIT', 'NETCDF3_CLASSIC'
#raw_forcing_format = 'SFX_NC' # files already in SFX format, 1 file per month
#FORC_VARS  = {'Tair':'Tair' ,'Qair':'Qair' ,'Wind':'Wind' ,'DIR_SWdown':'SWdown' ,'LWdown':'LWdown' ,'Rainf':'Prcp_ERAI' ,'Snowf':'Snow_ERAI' ,'PSurf':'PSurf'}
raw_forcing_format = 'VAR_NC' # seperated files for each variable (1 directory per variable), monthly files
FORC_VARS  = {'Tair':'Tair' ,'Qair':'Qair' ,'Wind':'Wind' ,'DIR_SWdown':'SWdown' ,'LWdown':'LWdown' ,'Rainf':'Rainf'     ,'Snowf':'Snowf'     ,'PSurf':'PSurf'}

# -------------------------------
# OBSERVATIONS
# -------------------------------
# Variables to be assimilated (names and folders)
obs_names                   = ['SWI'                            , 'LAI_V1'      ] # List of observation variables to be pre-processed.
obs_datasets                = ['CGLS'                           , 'CGLS_20D'    ] # Version of each observation variable
to_assim                    = [True                             , True          ] # Booleans indicating which observation variables (listed in obs_names) are assimilated in the analysis.
obs_from_pickle             = [True                             , True          ]
# Grids
obs_grids                   = ['def'                            , 'def'         ] # Observation grid types.
grid_transform_methods      = ['def'                            , 'def'         ] # Methods used to interpolate observation data on the model grid.
grid_transform_parameters   = ['def'                            , 'def'         ] # Parameters for interpolating observation data on the model grid.
# Screening with land surface properties
surf_screen_features        = [['urban_fraction', 'altitude']   , []            ] # Surface features used to screen observations (1 list per variable).
surf_screen_thresholds      = [[.15, 1500]                      , []            ] # Thresholds of surface features (listed in surf_screen_features) for discarding observations.
# Screening with model output variables
screen_mod_names            = [['WGI2']                         , []            ] # Model output variables used to screen observations (1 list per variable; the empty list [] means no screening).
screen_mod_steps            = [[0]                              , []            ] # Time steps (in range(mod_out_nsteps)) of model output used for screening.
screen_mod_thres            = [[0]                              , []            ] # Thresholds of model output variables (listed in screen_mod_names) for discarding observations.
# Re-scaling using model output variables
rescale_methods             = ['cdf_matching'                   , None          ] # Methods for re-scaling observations using model output (None means no re-scaling).
rescale_mod_names           = ['WG2'                            , None          ] # Model output variables used for re-scaling observations.
rescale_mod_steps           = [0                                , None          ] # Time steps (in range(mod_out_nsteps)) of model output used for re-scaling.
rescale_max_miss            = [.9                               , None          ] # Maximum fraction of missing observations for re-scaling a grid-point time series; if not satisfied, observations are set to missing.
rescale_calibs              = [True                             , None          ] # Booleans indicating whether the re-scaling parameters should be calibrated.
rescale_calib_periods       = [['2010-01-01', '2010-12-31']     , []            ] # Periods (start and end year, both included) over which the re-scaling parameters are calibrated.

# -------------------------------
# DIRECTORIES
# -------------------------------
# Input
options_path = exp_root + '/OPTIONS/'
mod_pgd_path = exp_root + '/pgd_prep/' # Must include PGD.nc, PGD_fractown.nc and PREP.nc
raw_forcing_dir = '/cnrm/vegeo/albergelc/Forcing/ERA5_025/' # Folder from where the raw forcing data is read
obs_root_dir = '/cnrm/vegeo/LDAS/OBSERVATIONS/' # Root folder of all observation files
openloop_dir = exp_root + '/results/ol/' # Folder from where openloop output is read in pre- and post-processing steps
analysis_dir = exp_root + '/results/'+exp_name+'/' # Folder from where analysis output is read in post-processing step
# Output
forcing_dir  = exp_root + '/forcings/' # Folder where NetCDF (as well as intermediate binary) forcing files are written.
out_obs_dir  = exp_root + '/observations/' # Folder where processed (CANARI files) observations are written
out_mod_dir  = exp_root + '/results/ldas_post/ol/' # Folder where PData (when asked) model output data are written
out_ana_dir  = exp_root + '/results/ldas_post/'+exp_name+'/' # Folder where PData (when asked) analysis output data are written
graphics_dir = exp_root + '/results/ldas_post/figures/' # Folder from where graphics files are written in the post-processing step.

# -------------------------------
# POST-PROCESSING
# -------------------------------
post_mod_from_pickle   = True # if True: try to read pickle files; if False or no pickle files: read model output files and save to pickle files.
post_ana_from_pickle   = True
post_obs_from_pickle   = True
post_zones             = {'def': None} # Default
extra_out_vars         = ['RUNOFFC']#,'RC_ECO'] # GPPC
trip_vars              = []
list_time_windows      = ['day' , 'month' , 'season' , 'year' , 'all']
var_spatial_maps_out   = [False , False   , True     , False  , True ]
var_time_series_out    = [True  , False   , False    , False  , False]
stats_spatial_maps_out = [False , False   , False    , False  , True ]
stats_time_series_out  = [False , False   , False    , True   , False]
stats_table_out        = [False , False   , False    , False  , False]
extra_diagnostics      = {'increments_assim'   : ['ANAL_INCR1'    , 'ANAL_INCR2'    ],\
                          'increments_names'   : ['LAI increments', 'WG2 increments'],\
                          'jacobian_assim'     : ['HO_1_1'   ,'HO_1_2'   ,'HO_2_1'   ,'HO_2_2'   ],\
                          'jacobian_names'     : ['dWG2/dLAI','dWG2/dWG2','dLAI/dLAI','dLAI/dWG2'],\
                          'patch'              : ['LAI']}
patch_out              = [4]#[4,5,6,7,8,9,10,11,12] # patch number between 1 and 12
modexp_names           = ['Open-Loop','Analysis'] # if 'def': ['Model','Analysis']
fig_dpi                = 300 # Resolution for saving raster figures.
# COPERNICUS figures to save to put in the final report
copernicus_figs        = False
#copernicus_fileForFigs = 'copernicus_figures.csv'


# Local tests
if False:
  list_time_windows      = ['all']
  var_spatial_maps_out   = [True    ]
  var_time_series_out    = [False   ]
  stats_spatial_maps_out = [False   ]
  stats_time_series_out  = [False   ]
  stats_table_out        = [False   ]
  #post_zones = {'point': [[-3.375,40.375]]}

