#!/usr/bin/python
import matplotlib as mpl
mpl.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from Tkinter import *
from tkFileDialog import *
from glob import glob
from ldasPost import *
#import myPyFunc as my
#plt = my.plt
#plt.ion()

# ------------------------------------------------------------------------------------------
#   MAIN INTERFACE
# ------------------------------------------------------------------------------------------

global data,opt
opt = dict()

root = Tk()
root.geometry('610x800+50+50')
root.winfo_toplevel().title("LDAS-Monde User Interface")

buttonQuit = Button(root, text='Exit', command = root.quit)
buttonQuit.place(x=550,y=760)


def new_row(frame):
  _,y = frame.grid_size()
  return y

def last_row(frame):
  _,y = frame.grid_size()
  return y-1

# ------------------------------------------------------------------------------------------
#   OPTIONS
# ------------------------------------------------------------------------------------------

def open_options_file(options_file=None):
  global opt
  if options_file is None:
    options_file = askopenfilename(title="Open options file",filetypes=[('options files','options*.py'),('all files','.*')])
  initPost(options_file)
  for v in ['assim_hour','patch_frac','post_from_pickle','map_set','obs_names']:
    exec('opt[\''+v+'\']='+v)
  opt['map_set'].fGUI = True
  if 'options_path' in globals():
    w_options_path.set(options_path)
  w_mod_pgd_path.set(mod_pgd_path)
  w_out_all_dir['Obs'].set(out_obs_dir)
  w_mod_ana_dir['Model'].set(openloop_dir)
  w_out_all_dir['Model'].set(out_mod_dir)
  w_mod_ana_dir['Analysis'].set(analysis_dir)
  w_out_all_dir['Analysis'].set(out_ana_dir)
  w_graphics_dir.set(graphics_dir)
  w_analysis_period[0].set(analysis_period[0])
  w_analysis_period[1].set(analysis_period[1])
  mod_zone = mod_grid['zone']
  w_map_extent[0][0].set(mod_zone[0][0])
  w_map_extent[0][1].set(mod_zone[0][1])
  w_map_extent[1][0].set(mod_zone[1][0])
  w_map_extent[1][1].set(mod_zone[1][1])

def clear_all_fields():
  for v in [w_options_path,w_mod_pgd_path,w_out_all_dir,w_mod_ana_dir,w_graphics_dir,\
            w_analysis_period,w_var,w_to_read,\
            w_ylim,w_clim]:
    if isinstance(v,dict):
      for vv in v.itervalues(): vv.set('')
    elif isinstance(v,list) or isinstance(v,tuple):
      for vv in v: vv.set('')
    else: v.set('')
    w_post_zone = 'default'
    w_to_read[0].set('0')
    w_to_read[1].set('1')
    w_to_read[2].set('1')

def browse_folder(which_variable):
  folder = askdirectory(initialdir=which_variable.get())
  which_variable.set(folder)

def options_add_dir(frame,text):
  y = new_row(frame)
  var = StringVar()
  Label(frame,text=text).grid(row=y+1,column=0,padx=2,sticky=W)
  Entry(frame,textvariable=var,bg='#fff',relief=FLAT,width=400).grid(row=y+1,column=1,sticky=EW,padx=2)
  Button(frame,image=img_folder,command=lambda: browse_folder(var)).grid(row=y+1,column=2,padx=2)
  return var

frameOptions = Frame(root,relief=GROOVE,bd=2)
frameOptions.place(x=5,y=5,width=600,height=260)

Button(frameOptions,text='Open options file',command=open_options_file).place(x=5,y=5)
Button(frameOptions,text='Clear',command=clear_all_fields).place(x=500,y=5)


# DIRECTORIES

frameDir = Frame(frameOptions)
frameDir.place(x=5,y=40,width=585,height=210)
frameDir.columnconfigure(1,weight=1)

img_folder = PhotoImage(file='images/folder20.png')
w_out_all_dir = dict()
w_mod_ana_dir = dict()

w_options_path = options_add_dir(frameDir,'OPTIONS.nam')
w_mod_pgd_path = options_add_dir(frameDir,'PGD')
w_out_all_dir['Obs'] = options_add_dir(frameDir,'Observations')
w_mod_ana_dir['Model'] = options_add_dir(frameDir,'Model')
w_mod_ana_dir['Analysis'] = options_add_dir(frameDir,'Analysis')
w_out_all_dir['Model'] = options_add_dir(frameDir,'Model PData')
w_out_all_dir['Analysis'] = options_add_dir(frameDir,'Analysis PData')
w_graphics_dir = options_add_dir(frameDir,'Figures')




# ------------------------------------------------------------------------------------------
#   READ DATA
# ------------------------------------------------------------------------------------------

def read_data():
  if w_var.get() == '': return
  global data,opt
  analysis_period = [w_analysis_period[0].get(),w_analysis_period[1].get()]
  to_read = [data_to_read[i] for i,w in enumerate(w_to_read) if w.get()=='1']
  out_all_dir = {d:w_out_all_dir[d].get() for d in to_read}
  mod_ana_dir = {d:w_mod_ana_dir[d].get() for d in to_read if d in ['Model','Analysis']}
  filter_nan = True if w_filter_nan.get()=='1' else False
  # post_zone: depending on w_post_zone
  if w_post_zone.get() == 'default':
    post_zones = {'def':None}
  elif w_post_zone.get() == 'rectangle':
    post_zones = {'zone':[[w_map_extent[0][0].get(),w_map_extent[0][1].get()],
                          [w_map_extent[1][0].get(),w_map_extent[1][1].get()]]}
  elif w_post_zone.get() == 'point':
    post_zones = {'point':[[w_map_point[0].get(),w_map_point[1].get()]]}
  elif w_post_zone.get() == 'mask':
    post_zones = {w_mask_name.get():w_mask_file.get()}
  post_zones = check_post_zones(post_zones)
  zname,zone = post_zones.iteritems().next()
  opt['map_set'],graphics_dir_zone,opt['IPNT'] = prepareZone(zname,zone,opt['map_set'],w_graphics_dir.get())
  data = readAllData(w_var.get(), analysis_period, opt['assim_hour'], opt['patch_frac'], \
                     out_all_dir, opt['post_from_pickle'], mod_ana_dir, to_read, IPNT=opt['IPNT'], \
                     filter_nan=filter_nan)

def create_pdata(mod):
  global opt
  var = w_var.get()
  if var in opt['obs_names']: var = OBS_MOD[var]
  period = [w_analysis_period[0].get(),w_analysis_period[1].get()]
  createPDataFromModel(var,period,opt['assim_hour'],opt['patch_frac'],\
                       w_mod_ana_dir[mod].get(),w_out_all_dir[mod].get())

def set_available_vars(*args):
  global opt
  if w_options_path.get() == '':
    w_var_entry.grid(row=row_var,column=1,sticky=EW,padx=2)
    w_var_menu.grid_remove()
  else:
    w_var_menu.grid(row=row_var,column=1,sticky=EW,padx=2)
    w_var_entry.grid_remove()
    menu = w_var_menu['menu']
    menu.delete(0,"end")
    obs_names = opt['obs_names']
    var_names = []
    for cselect_file in glob(w_options_path.get()+'CSELECT.*'):
      with open(cselect_file) as fid:
        f_cselect = False
        for l in fid.readlines():
          if 'CSELECT' in l: f_cselect = True
          if f_cselect:
            if len(l.split('\'')) < 2: continue
            var = l.split('\'')[-2]
            if not any([var==OBS_MOD[v] for v in obs_names]): var_names.append(var)
    for var in obs_names+var_names:
      menu.add_command(label=var,command=lambda value=var: w_var.set(value))
    w_var.set(menu.entrycget(0,'label'))

def activate_post_zone(*args):
  for f in [ff for zz,ff in frameZone.iteritems() if zz != w_post_zone.get()]:
    f.grid_remove()
  frameZone[w_post_zone.get()].grid(row=row_post_zone,column=0,columnspan=2,sticky=EW,padx=2)

def set_available_masks(*args):
  menu = w_mask_all_names['menu']
  menu.delete(0,"end")
  with Dataset(w_mask_file.get()) as nc:
    for name in nc.variables['Name'][:]:
      menu.add_command(label=name,command=lambda value=name: w_mask_name.set(value))

frameRead = Frame(root,relief=GROOVE,bd=2)#,bg='#888')
frameRead.place(x=5,y=270,width=200,height=350)
frameRead.columnconfigure(1,weight=1)

Label(frameRead,text='Data',font=('Times','14','bold italic')).grid(row=0,column=0,sticky=W)
Button(frameRead,text='Read',command=read_data).grid(row=0,column=1,padx=2,pady=2,sticky=E)

w_analysis_period = [StringVar(),StringVar()]
Label(frameRead,text='Period').grid(row=new_row(frameRead),column=0,rowspan=2,sticky=W)
Entry(frameRead,textvariable=w_analysis_period[0],bg='#fff',relief=FLAT,justify='center',width=10)\
  .grid(row=last_row(frameRead),column=1,sticky=W,padx=2)
Entry(frameRead,textvariable=w_analysis_period[1],bg='#fff',relief=FLAT,justify='center',width=10)\
  .grid(row=new_row(frameRead),column=1,sticky=W,padx=2)

row_var = new_row(frameRead)
w_var = StringVar()
Label(frameRead,text='Variable',justify='left').grid(row=row_var,column=0,sticky=W,padx=2)
w_var_entry = Entry(frameRead,textvariable=w_var,bg='#fff',relief=FLAT,width=10)
w_var_menu = OptionMenu(frameRead,w_var,'')
w_var_entry.grid(row=row_var,column=1,sticky=EW,padx=2)
w_options_path.trace('w',set_available_vars)

data_to_read,w_to_read = ['Obs','Model','Analysis'],[]
for i,d in enumerate(data_to_read):
  Label(frameRead,text=d,justify='left').grid(row=new_row(frameRead),column=0,sticky=W,padx=2)
  frameToread = Frame(frameRead)
  frameToread.grid(row=last_row(frameRead),column=1,sticky=EW,padx=2)
  w_to_read.append(StringVar(value=1*(i>0)))
  Checkbutton(frameToread,variable=w_to_read[i]).grid(row=0,column=0)
  if d in ['Model','Analysis']:
    Button(frameToread,text='Create PData',command=lambda: create_pdata(d),width=8).grid(row=0,column=1)

w_filter_nan = StringVar(value=1)
Label(frameRead,text='Filter NaN',justify='left').grid(row=new_row(frameRead),column=0,sticky=W,padx=2)
Checkbutton(frameRead,variable=w_filter_nan).grid(row=last_row(frameRead),column=1,sticky=W,padx=2)

w_post_zone = StringVar(value='default')
Label(frameRead,text='Post zone',justify='left').grid(row=new_row(frameRead),column=0,sticky=W,padx=2)
OptionMenu(frameRead,w_post_zone,'default','rectangle','point','mask').grid(row=last_row(frameRead),column=1,sticky=W,padx=2)

# === Zone ===

img_north = PhotoImage(file='images/north20.png')

frameZone = dict()
w_map_extent = [[StringVar(),StringVar()],[StringVar(),StringVar()]]
w_map_point = [StringVar(),StringVar()]
w_mask_name,w_mask_file = StringVar(),StringVar()

frameZone['default'] = Frame(frameRead)
for i in [0,1,2]: frameZone['default'].columnconfigure(i,weight=1)
Label(frameZone['default'],text='Model extent').grid(row=0,column=0,columnspan=3)
Label(frameZone['default'],image=img_north).grid(row=2,column=1)
Entry(frameZone['default'],textvariable=w_map_extent[0][0],justify='center',bg='#fff').grid(row=2,column=0)
Entry(frameZone['default'],textvariable=w_map_extent[0][1],justify='center',bg='#fff').grid(row=1,column=1)
Entry(frameZone['default'],textvariable=w_map_extent[1][0],justify='center',bg='#fff').grid(row=2,column=2)
Entry(frameZone['default'],textvariable=w_map_extent[1][1],justify='center',bg='#fff').grid(row=3,column=1)

frameZone['rectangle'] = Frame(frameRead)
for i in [0,1,2]: frameZone['rectangle'].columnconfigure(i,weight=1)
Label(frameZone['rectangle'],text='User defined rectangle').grid(row=0,column=0,columnspan=3)
Label(frameZone['rectangle'],image=img_north).grid(row=2,column=1)
Entry(frameZone['rectangle'],textvariable=w_map_extent[0][0],justify='center',bg='#fff').grid(row=2,column=0)
Entry(frameZone['rectangle'],textvariable=w_map_extent[0][1],justify='center',bg='#fff').grid(row=1,column=1)
Entry(frameZone['rectangle'],textvariable=w_map_extent[1][0],justify='center',bg='#fff').grid(row=2,column=2)
Entry(frameZone['rectangle'],textvariable=w_map_extent[1][1],justify='center',bg='#fff').grid(row=3,column=1)

frameZone['point'] = Frame(frameRead)
for i in [1,3]: frameZone['point'].columnconfigure(i,weight=1)
Label(frameZone['point'],text='Point').grid(row=0,column=0,columnspan=4)
Label(frameZone['point'],text='lon').grid(row=1,column=0)
Entry(frameZone['point'],textvariable=w_map_point[0],justify='center',bg='#fff').grid(row=1,column=1)
Label(frameZone['point'],text='lat').grid(row=1,column=2)
Entry(frameZone['point'],textvariable=w_map_point[1],justify='center',bg='#fff').grid(row=1,column=3)

frameZone['mask'] = Frame(frameRead)
frameZone['mask'].columnconfigure(1,weight=1)
Label(frameZone['mask'],text='Mask from netCDF',justify='left').grid(row=0,column=0,columnspan=3)
Label(frameZone['mask'],text='File',justify='left').grid(row=1,column=0)
Entry(frameZone['mask'],textvariable=w_mask_file,justify='left',bg='#fff').grid(row=1,column=1)
w_mask_file.trace('w',set_available_masks)
Button(frameZone['mask'],image=img_folder,command=lambda: w_mask_file.set(\
      askopenfilename(title="Open mask file",filetypes=[('mask files','*.nc'),('all files','.*')])))\
      .grid(row=1,column=2)
Label(frameZone['mask'],text='Name').grid(row=2,column=0)
w_mask_all_names = OptionMenu(frameZone['mask'],w_mask_name,'')
w_mask_all_names.grid(row=2,column=1,columnspan=2)

row_post_zone = new_row(frameRead)
w_post_zone.trace('w',activate_post_zone)
activate_post_zone()



# ------------------------------------------------------------------------------------------
#   NEW TIMESERIES
# ------------------------------------------------------------------------------------------

def draw_figure(fig):
  for t,f in fig.iteritems():
    new_fig = Toplevel(root)
    new_fig.title(t)
    canvas = FigureCanvasTkAgg(f, new_fig)
    canvas.show()
    canvas.get_tk_widget().pack(side=BOTTOM, fill=BOTH, expand=True)
    toolbar = NavigationToolbar2TkAgg(canvas, new_fig)
    toolbar.update()
    canvas._tkcanvas.pack(side=TOP, fill=BOTH, expand=True)

def new_plot_ts():
  global data
  data_tw = applyTimeWindow(data, w_time_window_ts.get())
  style = [w_line_style[c].get() for c in data_tw.items]
  minim = None if len(w_ylim[0].get())==0 else float(w_ylim[0].get())
  maxim = None if len(w_ylim[1].get())==0 else float(w_ylim[1].get())
  fig = draw_series_vars(data_tw.mean(axis=2),w_var.get(),style=style,minim=minim,maxim=maxim,\
                         time_window=w_time_window_ts.get(),fGUI=True)
  draw_figure(fig)

def new_plot_ts_stat():
  global data
  scores_for_all_points = compute_scores_for_all_points(data, w_time_window_ts.get())
  style = [w_line_style[c].get() for c in data.items]
  fig = draw_series_stats(scores_for_all_points,w_var.get(),time_window=w_time_window_ts.get(),fGUI=True)
  draw_figure(fig)

framePlot_ts = Frame(root,relief=GROOVE,bd=2)#,bg='#888')
framePlot_ts.place(x=210,y=270,width=195,height=350)
framePlot_ts.columnconfigure(1,weight=1)

Label(framePlot_ts,text='Time Series',font=('Times','14','bold italic')).grid(row=0,column=0,columnspan=2,sticky=W)
Button(framePlot_ts, text='Plot', command = new_plot_ts).grid(row=new_row(framePlot_ts),column=0,padx=2,pady=2,sticky=W)
Button(framePlot_ts, text='Stat', command = new_plot_ts_stat).grid(row=last_row(framePlot_ts),column=1,padx=2,pady=2,sticky=E)

w_time_window_ts = StringVar(value='day')
Label(framePlot_ts,text='Time window',justify='left').grid(row=new_row(framePlot_ts),column=0,sticky=W,padx=2)
OptionMenu(framePlot_ts,w_time_window_ts,'day','month','season','year','all').grid(row=last_row(framePlot_ts),column=1,sticky=W,padx=2)

w_ylim = [StringVar(),StringVar()]
Label(framePlot_ts,text='ylim',justify='left').grid(row=new_row(framePlot_ts),column=0,sticky=W,padx=2)
frameYlim = Frame(framePlot_ts)
frameYlim.grid(row=last_row(framePlot_ts),column=1,sticky=EW)
Entry(frameYlim,textvariable=w_ylim[0],bg='#fff',relief=FLAT,width=5).grid(row=0,column=0,padx=2)
Entry(frameYlim,textvariable=w_ylim[1],bg='#fff',relief=FLAT,width=5).grid(row=0,column=1,padx=2)

w_line_style = dict()
for i,d in enumerate(data_to_read):
  Label(framePlot_ts,text=d,justify='left').grid(row=new_row(framePlot_ts),column=0,sticky=W,padx=2)
  w_line_style[d] = StringVar(value=D_LINE[d])
  Entry(framePlot_ts,textvariable=w_line_style[d],bg='#fff',relief=FLAT,width=6).grid(row=last_row(framePlot_ts),column=1,padx=2,sticky=W)




# ------------------------------------------------------------------------------------------
#   NEW MAPS
# ------------------------------------------------------------------------------------------

def new_plot_map():
  global data,opt
  data_tw = applyTimeWindow(data, w_time_window_map.get())
  if w_plot_diff.get()=='1' and 'Model' in data_tw.items and 'Analysis' in data_tw.items:
      data_tw['Analysis-Model'] = data_tw['Analysis']-data_tw['Model']
      data_tw['Analysis-Model'].values[pl.where(data_tw['Analysis-Model'].values==0.)]=pl.nan
  minim = float(w_clim[0].get()) if w_clim[0].get() != '' else 0
  maxim = float(w_clim[1].get()) if w_clim[1].get() != '' else 0
  fig = draw_map_vars(opt['map_set'],w_var.get(),data_tw,time_window=w_time_window_map.get(),minim=minim,maxim=maxim)
  draw_figure(fig)

def new_plot_map_stat():
  global data,opt
  if not 'Obs' in data.items: return
  scores_per_point = compute_scores_per_point(data, w_time_window_map.get())
  fig = draw_map_stats(opt['map_set'],scores_per_point,obs_var=w_var.get(),time_window=w_time_window_map.get())
  draw_figure(fig)

framePlot_map = Frame(root,relief=GROOVE,bd=2)#,bg='#888')
framePlot_map.place(x=410,y=270,width=195,height=350)
framePlot_map.columnconfigure(1,weight=1)

Label(framePlot_map,text='Spatial Maps',font=('Times','14','bold italic')).grid(row=0,column=0,columnspan=2,sticky=W)
Button(framePlot_map, text='Plot', command = new_plot_map).grid(row=new_row(framePlot_map),column=0,padx=2,pady=2,sticky=W)
Button(framePlot_map, text='Stat', command = new_plot_map_stat).grid(row=last_row(framePlot_map),column=1,padx=2,pady=2,sticky=E)

w_time_window_map = StringVar(value='all')
Label(framePlot_map,text='Time window',justify='left').grid(row=new_row(framePlot_map),column=0,sticky=W,padx=2)
OptionMenu(framePlot_map,w_time_window_map,'month','season','year','all').grid(row=last_row(framePlot_map),column=1,sticky=W,padx=2)

w_clim = [StringVar(),StringVar()]
Label(framePlot_map,text='clim',justify='left').grid(row=new_row(framePlot_map),column=0,sticky=W,padx=2)
frameClim = Frame(framePlot_map)
frameClim.grid(row=last_row(framePlot_map),column=1,sticky=EW)
Entry(frameClim,textvariable=w_clim[0],bg='#fff',relief=FLAT,width=5).grid(row=0,column=0,padx=2)
Entry(frameClim,textvariable=w_clim[1],bg='#fff',relief=FLAT,width=5).grid(row=0,column=1,padx=2)

w_plot_diff = StringVar(value=1)
Label(framePlot_map,text='Plot diff',justify='left').grid(row=new_row(framePlot_map),column=0,sticky=W,padx=2)
Checkbutton(framePlot_map,variable=w_plot_diff).grid(row=last_row(framePlot_map),column=1,sticky=W,padx=2)




# ------------------------------------------------------------------------------------------
#   MAINLOOP
# ------------------------------------------------------------------------------------------

#open_options_file('options_Globe.py')
#read_data()
#new_plot_ts()
#new_plot_map()

root.mainloop()
#root.destroy()


