# ----------------------------------------------
# IMPORT PYTHON LIBRARIES
# ----------------------------------------------
from sys import argv
from csv import reader
from string import ascii_lowercase
from shutil import copyfile
from ldasModule import *
from ldasPost import *
from matplotlib.dates import MonthLocator, WeekdayLocator, DateFormatter
from matplotlib.colors import LinearSegmentedColormap
import matplotlib.gridspec as gridspec

# GRAPHICS
CBAR_MINMAX_INCR_last_vs_before = {'WG2':[-5e-3,5e-3],'LAI_V1':[-2e-2,2e-2]} # EU: 'WG2':[-5e-3,5e-3],'LAI':[-2e-2,2e-2]
CBAR_MINMAX_DIFF_last_vs_before = {'WG2':[-5e-3,5e-3],'WG3':[-2e-3,2e-3],'EVAPC':[-0.2,0.2],'DRAINC':[-0.2,0.2],\
                                   'RUNOFFC':[-2e-2,2e-2],'WGTOT':[-5e-3,5e-3],'GPPC':[-0.8,0.8],'RC_ECO':[-0.6,0.6],'NEEC':[-0.6,0.6],}

# ----------------------------------
# FUNCTIONS
# ----------------------------------

def readAllData_all3H_LST(period, assim_hour, patch_frac, out_all_dir, post_from_pickle, mod_ana_dir,\
			  to_read=['Model','Analysis','Obs']): 
    '''
    Very specific job for LST for COPERNICUS report: read all time steps of LST, every 3 hours, Model, Analysis and Observations
    '''
    # Initialization
    hours = pl.array([0,3,6,9,12,15,18,21])
    shift_val = -273.15
    # INIT
    IPNT = pl.ones(patch_frac.shape[1],bool)
    lst_dates = pd.date_range(pd.to_datetime(period[0]),pd.to_datetime(period[1])+pd.DateOffset(hour=23),freq='3H')
    data = pd.Panel(items=to_read, major_axis=lst_dates, minor_axis=pl.arange(len(IPNT))[IPNT],dtype='f4')
    #
    if 'Obs' in to_read:
        print('  ##OBS##')
        for hour in hours:
	    exist_pathPickles, list_pathPickles = findPDataFiles('LST_{0}'.format(hour), period, out_all_dir['Obs'])
            if post_from_pickle['Obs'] and any(exist_pathPickles):
                print('  Reading {0} observations at {1}H from PData files in {2}'.format('LST',hour,out_all_dir['Obs']))
		real_list_pathPickles = [list_pathPickles[i] for i in range(len(list_pathPickles)) if exist_pathPickles[i]]
		readPDataFiles(real_list_pathPickles, data['Obs'], IPNT)
        data['Obs'].values[:] += shift_val
    #
    if 'Model' in to_read:
        print('  ##MODEL##')
        for hour in hours:
	    exist_pathPickles, list_pathPickles = findPDataFiles('TS_{0}'.format(hour), period, out_all_dir['Model'])
            if post_from_pickle['Model'] and all(exist_pathPickles):
		print('  Reading {0} {1} from PData files in {2}'.format('TS_{0}'.format(hour),'Model',out_all_dir['Model']))
		readPDataFiles(list_pathPickles, data['Model'], IPNT)
	    else:
		# read data and create PData file 
		print('  Reading {0} {1} from SFX outputs in {2}'.format('TS_{0}'.format(hour),'Model',mod_ana_dir['Model']))
		data_tmp = readModOut('TS', patch_frac, mod_ana_dir['Model'], hour=hour, patch=None, NPNT=len(IPNT), assim_hour=assim_hour, period=period)
		if post_from_pickle['Model']: writePDataFiles('TS_{0}'.format(hour),data_tmp.index,data_tmp,out_all_dir['Model'])
		data['Model'].values[data.major_axis.hour==hour,:] = data_tmp.values
	data['Model'].values[:] += shift_val
    #
    if 'Analysis' in to_read:
        print('  ##ANALYSIS##')
        for hour in hours:
            exist_pathPickles, list_pathPickles = findPDataFiles('TS_{0}'.format(hour), period, out_all_dir['Analysis'])
            if post_from_pickle['Analysis'] and all(exist_pathPickles):
                print('  Reading {0} {1} from PData files in {2}'.format('TS_{0}'.format(hour),'Analysis',out_all_dir['Analysis']))
                readPDataFiles(list_pathPickles, data['Analysis'], IPNT)
            else:
                # read data and create PData file 
		print('  Reading {0} {1} from SFX outputs in {2}'.format('TS_{0}'.format(hour),'Analysis',mod_ana_dir['Analysis']))
		data_tmp = readModOut('TS', patch_frac, mod_ana_dir['Analysis'], hour=hour, patch=None, NPNT=len(IPNT), assim_hour=assim_hour, period=period)
		if post_from_pickle['Analysis']: writePDataFiles('TS_{0}'.format(hour),data_tmp.index,data_tmp,out_all_dir['Analysis'])
		data['Analysis'].values[data.major_axis.hour==hour,:] = data_tmp.values
	data['Analysis'].values[:] += shift_val
    # Remove nan values
    data = remove_nan(data)
    return data
#
def draw_series_bias_LST_all3H(data, graphics_dir, fig_dpi):
    #
    data_aveAllPoints = data.mean(2)
    #bias = pd.DataFrame(pl.empty((2,len(data_aveAllPoints['Model'])))*pl.nan,['Model','Analysis'],data_aveAllPoints.index)
    hours = pl.unique(data_aveAllPoints.index.hour)
    #
    for month in range(1,13):
        bias_3H = pd.DataFrame(pl.empty((len(hours),2))*pl.nan,hours,['lastyear','before'])
        id_in_month = pl.find(data_aveAllPoints.index.month==month)
        for h in hours:
            id_h_lastyear = pl.find(pl.logical_and(data_aveAllPoints['Obs'][id_in_month].index.hour==h,\
                                    data_aveAllPoints['Obs'][id_in_month].index.year==data_aveAllPoints['Obs'][id_in_month].index.year[-1]))
            id_h_before   = pl.find(pl.logical_and(data_aveAllPoints['Obs'][id_in_month].index.hour==h,\
                                    pl.logical_and(data_aveAllPoints['Obs'][id_in_month].index.year<data_aveAllPoints['Obs'][id_in_month].index.year[-1],\
                                                   data_aveAllPoints['Obs'][id_in_month].index.year>=2010)))
            bias_3H['lastyear'][h] = pl.mean(data_aveAllPoints['Analysis'][id_in_month[id_h_lastyear]]-\
                                             data_aveAllPoints['Obs'][id_in_month[id_h_lastyear]])
            bias_3H['before'][h]   = pl.mean(data_aveAllPoints['Analysis'][id_in_month[id_h_before]]-\
                                             data_aveAllPoints['Obs'][id_in_month[id_h_before]])
        fig = pl.figure()
        ax  = fig.add_subplot(111)
        bias_3H.plot(ax=ax,xticks=hours,xlim=hours[[0,-1]]+pl.arange(-1,2,2))
        ax.set_xlabel('Hour')
        ax.set_ylabel(' '.join((VAR_YLAB['LST'],'bias',VAR_UNIT['LST'])))
        ax.set_title(pd.datetools.MONTHS[month-1])
        lgd_lastyear = '{0}'.format(pl.unique(data_aveAllPoints['Obs'][id_in_month[id_h_lastyear]].index.year)[-1])
        lgd_before   = '{0} to {1}'.format(pl.unique(data_aveAllPoints['Obs'][id_in_month[id_h_before]].index.year)[0],\
                                           pl.unique(data_aveAllPoints['Obs'][id_in_month[id_h_before]].index.year)[-1])
        ax.legend((lgd_lastyear,lgd_before))
        fig_name = os.path.normpath('{0}/LST_hourly_bias_{1}_{2}_to_{3}_lastyear_vs_before.png'.format(graphics_dir,pd.datetools.MONTHS[month-1],\
                                pl.unique(data_aveAllPoints['Obs'][id_in_month[id_h_before]].index.year)[0],\
                                pl.unique(data_aveAllPoints['Obs'][id_in_month[id_h_lastyear]].index.year)[-1]))
        fig.savefig(fig_name, dpi=fig_dpi, bbox_inches='tight')
        print(fig_name)
        pl.close()
#
def draw_series_stats_lastyear_vs_before(scores_lastyear, scores_vsbefore, time_window, obs, graphics_dir, fig_dpi, lbl_vsbefore, lbl_lastyear, scores_eachY=None):
    '''
    Draws time series of scores of last year vs. before
    '''
    # To display
    diagnos2show = scores_lastyear.major_axis
    if 'Analysis-Model' in diagnos2show: diagnos2show=diagnos2show.drop('Analysis-Model')
    # 2 cases:
    if isinstance(scores_lastyear.minor_axis[0],str): #time_window=='season'
        fig_name = os.path.normpath('{0}/{1}_{2}_scores_{3}_to_{4}_lastyear_vs_before.png'.\
                                    format(graphics_dir, obs, TW_USER_FIG[time_window], \
                                    scores_vsbefore.minor_axis[0], scores_lastyear.minor_axis[-1]))
        t = range(0, scores_lastyear.minor_axis.size)
        xlim = [-.25, scores_lastyear.minor_axis.size-.75]
        xtickpos = range(scores_lastyear.minor_axis.size)
        xticklbl = [m[0:3] for m in scores_lastyear.minor_axis]
    else:
        fig_name = os.path.normpath('{0}/{1}_{2}_scores_{3}_to_{4}_lastyear_vs_before.png'.\
                                    format(graphics_dir, obs, TW_USER_FIG[time_window], \
                                    scores_lastyear.minor_axis[0].strftime(TW_USER_STRFTIME[time_window]), \
                                    scores_lastyear.minor_axis[-1].strftime(TW_USER_STRFTIME[time_window])))
        t = scores_lastyear.minor_axis.to_datetime()
        xlim = [t[0]-pd.DateOffset(months=1), t[-1]+pd.DateOffset(months=1)]
        xtickpos = pd.date_range(t[0]-pd.DateOffset(months=1),t[-1]+pd.DateOffset(months=1),freq='A')+pd.DateOffset(days=1)
        xticklbl = [xtickpos[i].strftime('%Y') for i in range(len(xtickpos))]
    # Plot
    #gs = gridspec.GridSpec(5,2)
    fig = pl.figure(figsize=[12,9+5]) #[l/3.*4.5 for l in pl.rcParams['figure.figsize']]
    for sco, i in zip(list(set(SCORES)-set(['Nb_pts'])),range(len(list(set(SCORES)-set(['Nb_pts']))))):
        ax = fig.add_subplot(3,2,i+1)
        title_str = sco if sco == 'Correlation' else ' '.join((sco,VAR_UNIT[obs]))
        ph   = []
        labs = []
        for diagno in diagnos2show:
            if scores_eachY: 
                #for yy in range(len(scores_eachY)): pl.plot(t, scores_eachY[yy][sco].ix[diagno], color=D_LINE[diagno], linestyle='--', alpha=0.25)
                min_sco,max_sco = pl.ones((len(t),))*pl.inf,pl.ones((len(t),))*-pl.inf
                for tt in range(len(t)):
                    for yy in range(len(scores_eachY)-1): 
                        min_sco[tt] = min(min_sco[tt],scores_eachY[yy][sco].ix[diagno][tt])
                        max_sco[tt] = max(max_sco[tt],scores_eachY[yy][sco].ix[diagno][tt])
                p_tmp = ax.fill_between(t, min_sco, max_sco, facecolor=D_LINE[diagno],alpha=0.1,interpolate=True, label='min/max '+diagno+' '+lbl_vsbefore)
                ph.append(p_tmp) 
                labs.append(p_tmp.get_label())
            p_tmp = ax.plot(t, scores_lastyear[sco].ix[diagno], color=D_LINE[diagno], linestyle='-' , label=diagno+' '+lbl_lastyear, lw=2)
            ph.append(p_tmp[0])
            labs.append(p_tmp[0].get_label())
            p_tmp = ax.plot(t, scores_vsbefore[sco].ix[diagno], color=D_LINE[diagno], linestyle='--', label=diagno+' '+lbl_vsbefore, lw=2)
            ph.append(p_tmp[0])
            labs.append(p_tmp[0].get_label())
        #labs = [ph[i][0].get_label() for i in range(3*len(diagnos2show))]
        ax.set_title(title_str)
        ax.set_xlim(xlim)
        ax.set_xticks(xtickpos)
        ax.set_xticklabels(xticklbl)
        ax.grid()
    #fig.legend(list(ph[id_y][0] for id_y in range(6)), labs, ncol=len(diagnos2show)+1, loc=8, bbox_to_anchor=(.0725, 0, .75, .95), borderaxespad=0.)
    #fig.legend(list(ph), labs, ncol=len(diagnos2show), loc=8, bbox_to_anchor=(.0725, 0, .75, .95), borderaxespad=0.)
    fig.legend(list(ph), labs, ncol=len(diagnos2show), loc='center', bbox_to_anchor=(0.46,0.175), borderaxespad=0.)
    #fig.legend(ax.lines, labs, ncol=len(diagnos2show), loc=8, bbox_to_anchor=(.0725, 0, .75, .95), borderaxespad=0.)
    # Save figure
    fig.savefig(fig_name, dpi=fig_dpi, bbox_inches='tight')
    print(fig_name)
    pl.close()
    # 
    # Tables of number of data only 
    '''
    n_rows = 2 
    n_cols = 3+len(scores_lastyear.minor_axis)
    col_labels = pl.hstack((['Period'],scores_lastyear.minor_axis,['min','max']))
    row_period = [lbl_vsbefore,lbl_lastyear]
    table2draw  = n_rows*[[]]
    table2draw[0] = [lbl_vsbefore] + ['{:.0f}'.format(scores_vsbefore['Nb_pts'].values[0,i]) for i in range(len(scores_lastyear.minor_axis))] +\
                    ['{:.0f}'.format(min(scores_vsbefore['Nb_pts'].values[0,:])), '{:.0f}'.format(max(scores_vsbefore['Nb_pts'].values[0,:]))]
    table2draw[1] = [lbl_lastyear] + ['{:.0f}'.format(scores_lastyear['Nb_pts'].values[0,i]) for i in range(len(scores_lastyear.minor_axis))] +\
                    ['{:.0f}'.format(min(scores_lastyear['Nb_pts'].values[0,:])), '{:.0f}'.format(max(scores_lastyear['Nb_pts'].values[0,:]))]
    # Figure
    fig = pl.figure(figsize=(1.*n_cols, .3*(n_rows+1)))
    ax = fig.add_subplot(111)
    ax.axis('off')
    the_table = ax.table(cellText=table2draw, colLabels=col_labels, loc='center')
    for r in range(n_rows+1):
        for c in range(n_cols):
            cell = the_table.properties()['celld'][r,c]
            cell.set_fontsize(20)
    #cell = the_table.properties()['celld'][0,c]
    #cell.set_height(2.5*cell.get_height())
    pl.show()
    '''
    n_rows = 2
    n_cols = 2+len(scores_lastyear.minor_axis)
    col_labels = pl.hstack((scores_lastyear.minor_axis,['min','max']))
    row_labels = pl.array([lbl_vsbefore,lbl_lastyear],dtype=object)
    table2draw  = n_rows*[[]]
    table2draw[0] = ['{:.0f}'.format(scores_vsbefore['Nb_pts'].values[0,i]) for i in range(len(scores_lastyear.minor_axis))] +\
                    ['{:.0f}'.format(min(scores_vsbefore['Nb_pts'].values[0,:])), '{:.0f}'.format(max(scores_vsbefore['Nb_pts'].values[0,:]))]
    table2draw[1] = ['{:.0f}'.format(scores_lastyear['Nb_pts'].values[0,i]) for i in range(len(scores_lastyear.minor_axis))] +\
                    ['{:.0f}'.format(min(scores_lastyear['Nb_pts'].values[0,:])), '{:.0f}'.format(max(scores_lastyear['Nb_pts'].values[0,:]))]
    # Figure
    fig = pl.figure(figsize=(1.*n_cols, .3*(n_rows+1)))
    ax = fig.add_subplot(111)
    ax.axis('off')
    the_table = ax.table(cellText=table2draw, rowLabels=row_labels, colLabels=col_labels, loc='center', colWidths=[0.05] * 14)
    #
    if isinstance(scores_lastyear.minor_axis[0],str): #time_window=='season'
        fig_name = os.path.normpath('{0}/{1}_{2}_nbObs_{3}_to_{4}_lastyear_vs_before.png'.\
                                    format(graphics_dir, obs, TW_USER_FIG[time_window], \
                                    scores_vsbefore.minor_axis[0], scores_lastyear.minor_axis[-1]))
    else:
        fig_name = os.path.normpath('{0}/{1}_{2}_nbObs_{3}_to_{4}_lastyear_vs_before.png'.\
                                    format(graphics_dir, obs, TW_USER_FIG[time_window], \
                                    scores_lastyear.minor_axis[0].strftime(TW_USER_STRFTIME[time_window]), \
                                    scores_lastyear.minor_axis[-1].strftime(TW_USER_STRFTIME[time_window])))
    fig.savefig(fig_name, dpi=fig_dpi, bbox_inches='tight')
    print(fig_name)
    pl.close()
#
def drawScoresTable_lastyear_vs_before(scores_lastyear, scores_vsbefore, time_window, obs, graphics_dir, fig_dpi, lbl_vsbefore, lbl_lastyear):
    '''
    Draws tables of scores for 'Model' and 'Analysis' for last year vs all the years before
    '''
    # Construct the table with the scores to save
    '''
    n_rows = 4
    n_cols = 2+len(scores_lastyear.items)       # period, version, SCORES
    col_labels = pl.hstack((['Period','Version'],scores_lastyear.items))
    col_period = [lbl_vsbefore, lbl_vsbefore, lbl_lastyear, lbl_lastyear]
    col_version = scores_lastyear.major_axis[0:2].repeat(n_rows/2).reshape((2,n_rows/2)).T.ravel()
    table2draw  = n_rows*[[]]
    table2draw[0] = [lbl_vsbefore, 'Model']    + ['{:.3f}'.format(scores_vsbefore.values[i,0,0]) for i in range(len(scores_vsbefore.items)-1)] + \
                                                 ['{:.0f}'.format(scores_vsbefore['Nb_pts'].values[0][0])]
    table2draw[1] = [lbl_vsbefore, 'Analysis'] + ['{:.3f}'.format(scores_vsbefore.values[i,1,0]) for i in range(len(scores_vsbefore.items)-1)] + \
                                                 ['{:.0f}'.format(scores_vsbefore['Nb_pts'].values[1][0])]
    table2draw[2] = [lbl_lastyear, 'Model']    + ['{:.3f}'.format(scores_lastyear.values[i,0,0]) for i in range(len(scores_lastyear.items)-1)] + \
                                                 ['{:.0f}'.format(scores_lastyear['Nb_pts'].values[0][0])]
    table2draw[3] = [lbl_lastyear, 'Analysis'] + ['{:.3f}'.format(scores_lastyear.values[i,1,0]) for i in range(len(scores_lastyear.items)-1)] + \
                                                 ['{:.0f}'.format(scores_lastyear['Nb_pts'].values[1][0])]
    '''
    n_rows = 4 
    n_cols = 1+len(scores_lastyear.items)
    col_labels = pl.hstack((['Version'],scores_lastyear.items))
    row_labels = [lbl_vsbefore, lbl_vsbefore, lbl_lastyear, lbl_lastyear]
    table2draw  = n_rows*[[]]
    table2draw[0] = ['Model']    + ['{:.3f}'.format(scores_vsbefore.values[i,0,0]) for i in range(len(scores_vsbefore.items)-1)] + \
		                   ['{:.0f}'.format(scores_vsbefore['Nb_pts'].values[0][0])]
    table2draw[1] = ['Analysis'] + ['{:.3f}'.format(scores_vsbefore.values[i,1,0]) for i in range(len(scores_vsbefore.items)-1)] + \
                                   ['{:.0f}'.format(scores_vsbefore['Nb_pts'].values[1][0])]
    table2draw[2] = ['Model']    + ['{:.3f}'.format(scores_lastyear.values[i,0,0]) for i in range(len(scores_lastyear.items)-1)] + \
		                   ['{:.0f}'.format(scores_lastyear['Nb_pts'].values[0][0])]
    table2draw[3] = ['Analysis'] + ['{:.3f}'.format(scores_lastyear.values[i,1,0]) for i in range(len(scores_lastyear.items)-1)] + \
		                   ['{:.0f}'.format(scores_lastyear['Nb_pts'].values[1][0])]
    # Figure
    fig = pl.figure(figsize=(1.*n_cols, .3*(n_rows+1)))
    ax = fig.add_subplot(111)
    ax.axis('off')
    the_table = ax.table(cellText=table2draw, rowLabels=row_labels, colLabels=col_labels, loc='center', colWidths=[0.05]*6)
    fig.subplots_adjust(top = 1, bottom = 0, right = 1, left = 0)#, hspace = 0, wspace = 0)
    #ax.margins(0,0)
    #ax.xaxis.set_major_locator(pl.NullLocator())
    #ax.yaxis.set_major_locator(pl.NullLocator())
    #fig.savefig('toto.png', dpi=fig_dpi, bbox_inches='tight')
    #
    '''
    fig = pl.figure(figsize=(1.*n_cols, .3*(n_rows+1)))
    ax = fig.add_subplot(111)
    ax.axis('off')
    the_table = ax.table(cellText=table2draw, colLabels=col_labels, loc='center', fontsize=20)
    #for r in range(n_rows+1):
    #    for c in range(n_cols):
    #            cell = the_table.properties()['celld'][0,c]
    #       cell.set_height(2.5*cell.get_height())
    #            cell = the_table.properties()['celld'][r,c]
    #            cell.set_fontsize(20)
    '''
    fig_name = os.path.normpath('{0}/{1}_{2}_table_scores_{3}_to_{4}_lastyear_vs_before.png'.\
                                format(graphics_dir, obs, TW_USER_FIG[time_window], \
                                scores_vsbefore.minor_axis[0][0:6], scores_lastyear.minor_axis[0][-6:]))
    # Save figure
    fig.savefig(fig_name, dpi=fig_dpi, bbox_inches='tight')
    print(fig_name)
    pl.close()
#
#
def draw_series_stats_SPOTVGT_PROBAV(scores_for_all_points_SPOTVGT, scores_for_all_points_PROBAV, time_window, obs, graphics_dir, fig_dpi):
    '''
    Draws time series of scores of last year vs. before
    '''
    # To display
    diagnos2show = scores_for_all_points_SPOTVGT.major_axis
    if 'Analysis-Model' in diagnos2show: diagnos2show=diagnos2show.drop('Analysis-Model')
    # 2 cases:
    if isinstance(scores_for_all_points_SPOTVGT.minor_axis[0],str): #time_window=='season'
        fig_name = os.path.normpath('{0}/{1}_{2}_scores_SPOTVGT_vs_PROBAV.png'.format(graphics_dir, obs, TW_USER_FIG[time_window],))
        t = range(0, scores_for_all_points_SPOTVGT.minor_axis.size)
        xlim = [-.25, scores_for_all_points_SPOTVGT.minor_axis.size-.75]
        xtickpos = range(scores_for_all_points_SPOTVGT.minor_axis.size)
        xticklbl = [m[0:3] for m in scores_for_all_points_SPOTVGT.minor_axis]
    else:
        fig_name = os.path.normpath('{0}/{1}_{2}_scores_SPOTVGT_vs_PROBAV.png'.format(graphics_dir, obs, TW_USER_FIG[time_window],))
        t = scores_for_all_points_SPOTVGT.minor_axis.to_datetime()
        xlim = [t[0]-pd.DateOffset(months=1), t[-1]+pd.DateOffset(months=1)]
        xtickpos = pd.date_range(t[0]-pd.DateOffset(months=1),t[-1]+pd.DateOffset(months=1),freq='A')+pd.DateOffset(days=1)
        xticklbl = [xtickpos[i].strftime('%Y') for i in range(len(xtickpos))]
    # Plot
    fig = pl.figure(figsize=[l/3.*4.5 for l in pl.rcParams['figure.figsize']])
    for sco, i in zip(list(set(SCORES)-set(['Nb_pts'])),range(len(list(set(SCORES)-set(['Nb_pts']))))):
        ax = fig.add_subplot(2,2,i+1)
        title_str = sco if sco == 'Correlation' else ' '.join((sco,VAR_UNIT[obs]))
        ph = []
        for diagno in diagnos2show:
            p_tmp = ax.plot(t, scores_for_all_points_PROBAV[sco].ix[diagno],  color=D_LINE[diagno], linestyle='-' , label=diagno+' PROBA-V')
            ph.append(p_tmp)
            p_tmp = ax.plot(t, scores_for_all_points_SPOTVGT[sco].ix[diagno], color=D_LINE[diagno], linestyle='--', label=diagno+' SPOT-VGT')
            ph.append(p_tmp)
        labs = [ph[i][0].get_label() for i in range(2*len(diagnos2show))]
        ax.set_title(title_str)
        ax.set_xlim(xlim)
        ax.set_xticks(xtickpos)
        ax.set_xticklabels(xticklbl)
        ax.grid()
    fig.legend(ax.lines, labs, ncol=len(diagnos2show), loc=8, bbox_to_anchor=(.0725, 0, .75, .95), borderaxespad=0.)
    # Save figure
    fig.savefig(fig_name, dpi=fig_dpi, bbox_inches='tight')
    print(fig_name)
    pl.close()
# 

def postObs_last_year_vs_before(obs, data=None, minim=None, maxim=None):
    importLocals(currentframe())
    print('===================================================')
    print('Post-processing for observation: {0}'.format(obs)+(', model variable: {1}'.format(obs, OBS_MOD[obs]) if obs in OBS_MOD.keys() else ''))
    if data is None: 
        data = readAllData(obs, analysis_period, assim_hour, patch_frac, out_all_dir, \
                    post_from_pickle, mod_ana_dir, ['Model','Analysis','Obs'], IPNT=IPNT, patch=None, map_set=map_set)
    if 0 in data.shape:
        print('No dataset or no time step or no point. Continuing with next obs...')
        return
    for time_window,t in zip(list_time_windows,range(len(list_time_windows))): 
        # Compute statistics for last year vs all the others before: (SA is not available after May 2014)
        if 'Obs' in data.items and obs!='SA' and (time_window=='season' or time_window=='all'): # and obs!='SIF' and (time_window=='season' or time_window=='all')
            print('Statistics are computed for last year vs. before for {0} at scale {1}'.format(obs,time_window))
            if 'SIF' in obs: 
                scores_for_all_points_lastyear = compute_scores_for_all_points(applyTimeWindow(data.ix[:,(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y-%m-%d'):pd.to_datetime(analysis_period[1]).strftime('%Y-%m-%d'),:],'month'), time_window)
                scores_for_all_points_vsbefore = compute_scores_for_all_points(applyTimeWindow(data.ix[:, pd.to_datetime(analysis_period[0]).strftime('%Y-%m-%d'):(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(hours=23)).strftime('%Y-%m-%d'),:],'month'),time_window)
                if time_window=='season': scores_for_all_points_eachY = [compute_scores_for_all_points(applyTimeWindow(data.ix[:,'{0}-01-01'.format(str(yy)):'{0}-12-31'.format(str(yy)),:],'month'), time_window) for yy in pl.arange(data.major_axis.year[0],data.major_axis.year[-1]+1)]
            else:
                scores_for_all_points_lastyear = compute_scores_for_all_points(data.ix[:,(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y-%m-%d'):pd.to_datetime(analysis_period[1]).strftime('%Y-%m-%d'),:], time_window)
                scores_for_all_points_vsbefore = compute_scores_for_all_points(data.ix[:, pd.to_datetime(analysis_period[0]).strftime('%Y-%m-%d'):(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(hours=23)).strftime('%Y-%m-%d'),:],time_window)
                if time_window=='season': scores_for_all_points_eachY = [compute_scores_for_all_points(data.ix[:,'{0}-01-01'.format(str(yy)):'{0}-12-31'.format(str(yy)),:], time_window) for yy in pl.arange(data.major_axis.year[0],data.major_axis.year[-1]+1)]
                #scores_for_all_points_eachY = pd.Panel4D({str(data.major_axis.year[0]):compute_scores_for_all_points(data.ix[:,'{0}-01-01'.format(data.major_axis.year[0]):'{0}-12-31'.format(data.major_axis.year[0]),:], time_window)})
                #for yy in pl.arange(data.major_axis.year[0]+1,data.major_axis.year[-1]+1): 
                #    scores_for_all_points_eachY.join(pd.Panel4D({str(yy):compute_scores_for_all_points(data.ix[:,'{0}-01-01'.format(yy):'{0}-12-31'.format(yy),:], time_window)}))
            #
	    if time_window=='season':
                draw_series_stats_lastyear_vs_before(scores_for_all_points_lastyear, scores_for_all_points_vsbefore, time_window, obs, graphics_dir_zone, fig_dpi,'{0} - {1}'.format(pd.to_datetime(analysis_period[0]).strftime('%Y-%m-%d'),(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)).strftime('%Y-%m-%d')),'{0} - {1}'.format((pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y-%m-%d'),pd.to_datetime(analysis_period[1]).strftime('%Y-%m-%d')), scores_eachY=scores_for_all_points_eachY)
            elif time_window=='all':
		drawScoresTable_lastyear_vs_before(scores_for_all_points_lastyear, scores_for_all_points_vsbefore, time_window, obs, graphics_dir_zone, fig_dpi,'{0} - {1}'.format(pd.to_datetime(analysis_period[0]).strftime('%Y-%m-%d'),(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)).strftime('%Y-%m-%d')),'{0} - {1}'.format((pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y-%m-%d'),pd.to_datetime(analysis_period[1]).strftime('%Y-%m-%d')))


def postIncr_last_year_vs_before():
    importLocals(currentframe())
    print('===================================================')
    print('Reading and plotting increments of assimilation')
    # Read increments
    incr = readAllAnalysis(extra_diagnostics['increments_assim'], analysis_period, assim_hour, patch_frac, out_all_dir, post_from_pickle, mod_ana_dir, IPNT=IPNT, filter_nan=False)
    if 0 in incr.shape:
        print('No dataset or no time step or no point. Continuing with next obs...')
        return
    for time_window,t in zip(list_time_windows,range(len(list_time_windows))) :
        if time_window=='season':
            incr_tw_lastyear = applyTimeWindow(incr.ix[:,(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y-%m-%d'):pd.to_datetime(analysis_period[1]).strftime('%Y-%m-%d'),:], time_window)
            incr_tw_vsbefore = applyTimeWindow(incr.ix[:, pd.to_datetime(analysis_period[0]).strftime('%Y-%m-%d'):(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(hours=23)).strftime('%Y-%m-%d'),:], time_window)
            for incr_x,incr_name,k in zip(incr_tw_lastyear.items,extra_diagnostics['increments_names'],range(len(incr_tw_lastyear.items))):
                # short incr_name
                id_ = incr_name.rfind(' ')
                if id_ != -1: incr_name=incr_name[0:id_]
                incr_tw = pd.Panel(pl.nan*pl.empty((2,len(incr_tw_lastyear.major_axis),len(incr_tw_lastyear.minor_axis))),\
                    [pd.to_datetime(analysis_period[0]).strftime('%Y')+':'+(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)).strftime('%Y'),\
                     (pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y')],\
                    incr_tw_lastyear.major_axis, incr_tw_lastyear.minor_axis)
                incr_tw[pd.to_datetime(analysis_period[0]).strftime('%Y')+':'+(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)).strftime('%Y')] = incr_tw_vsbefore[incr_x]
                incr_tw[(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y')]                                 = incr_tw_lastyear[incr_x]
                incr_tw.values[pl.where(incr_tw==0.)]=pl.nan
                # Draw maps
                minim,maxim = 0,0
	        #if incr_name in CBAR_MINMAX_INCR_last_vs_before: 
		#    minim = CBAR_MINMAX_INCR_last_vs_before[incr_name][0]
		#    maxim = CBAR_MINMAX_INCR_last_vs_before[incr_name][1]
                if 'increments_lim_vs' in extra_diagnostics.keys():
                    minim = extra_diagnostics['increments_lim_vs'][k][0]
                    maxim = extra_diagnostics['increments_lim_vs'][k][1]
                draw_map_analysis(map_set, incr_tw, incr_tw.items, graphics_dir_zone,\
                                  incr_name+'_increments_lastyear_vs_before', time_window, None, \
                                  fig_dpi, minim=minim, maxim=maxim)


def postExtraVar_last_year_vs_before(var, data=None, minim=None, maxim=None):
    importLocals(currentframe())
    print('===================================================')
    print('Post-processing for model variable {0}'.format(var))
    # Read all data
    if data is None:
        if var in ['RC_ECO','NEEC']:
            # NEEC = RC_ECO - GPPC (RC_ECO needs to be corrected also)
            data_RC_ECO = readAllData('RC_ECO', analysis_period, assim_hour, patch_frac, \
                            out_all_dir, post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT)
            data_GPPC   = readAllData('GPPC'  , analysis_period, assim_hour, patch_frac, \
                            out_all_dir, post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT)
            # needs to apply a correction for the NIT option (*0.72*(sum_time(GPPC)/sum_time(RC_ECO)))
            for i in range(2): data_RC_ECO.values[i,:] = 0.72*data_RC_ECO.values[i,:]*pl.nansum(data_GPPC.values[i,:],axis=0)/pl.nansum(data_RC_ECO.values[i,:],axis=0)
            #
            data = data_RC_ECO.copy()
            if var=='NEEC': data.values[:] = data.values[:] - data_GPPC.values[:]
            del data_RC_ECO, data_GPPC
        else:
            data = readAllData(var, analysis_period, assim_hour, patch_frac, \
                           out_all_dir, post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT)
    #
    '''
    if data is None:
	if var=='NEEC':
            # NEEC = RC_ECO - GPPC (RC_ECO needs to be corrected also)
            data_RC_ECO = readAllData('RC_ECO', analysis_period, assim_hour, patch_frac, \
                            out_all_dir, post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT)
            data_GPPC   = readAllData('GPPC'  , analysis_period, assim_hour, patch_frac, \
                            out_all_dir, post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT)
            # needs to apply a correction for the NIT option (*0.72*(sum_time(GPPC)/sum_time(RC_ECO)))
            for i in range(2): data_RC_ECO.values[i,:] = 0.72*data_RC_ECO.values[i,:]*pl.nansum(data_GPPC.values[i,:],axis=0)/pl.nansum(data_RC_ECO.values[i,:],axis=0)
            #
            if IPNT is None: IPNT_tmp = pl.ones(patch_frac.shape[1],bool)
            else:            IPNT_tmp = IPNT.copy()
            data = pd.Panel(items=['Model','Analysis'],major_axis=data_RC_ECO.major_axis,minor_axis=pl.arange(len(IPNT_tmp))[IPNT_tmp],dtype='f4')
            data.values[:] = data_RC_ECO.values[:] - data_GPPC.values[:]
            del IPNT_tmp, data_RC_ECO, data_GPPC
	else:
            data = readAllData(var, analysis_period, assim_hour, patch_frac, out_all_dir, \
                           post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT, patch=None)
	    if var=='RC_ECO':
                # needs to apply a correction for the NIT option (*0.72*(sum_time(GPPC)/sum_time(RC_ECO)))
                data_GPPC = readAllData('GPPC'  , analysis_period, assim_hour, patch_frac, \
                            out_all_dir, post_from_pickle, mod_ana_dir, ['Model','Analysis'], IPNT=IPNT)
                for i in range(2): data.values[i,:] = 0.72*data.values[i,:]*pl.nansum(data_GPPC.values[i,:],axis=0)/pl.nansum(data.values[i,:],axis=0)
                del data_GPPC
    '''
    #
    if 0 in data.shape:
        print('No dataset or no time step or no point. Continuing with next obs...')
        return
    for time_window,t in zip(list_time_windows,range(len(list_time_windows))):
        if time_window=='season':
            data_tw_lastyear = applyTimeWindow(data.ix[:,(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y-%m-%d'):pd.to_datetime(analysis_period[1]).strftime('%Y-%m-%d'),:], time_window)
            data_tw_vsbefore = applyTimeWindow(data.ix[:, pd.to_datetime(analysis_period[0]).strftime('%Y-%m-%d'):(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(hours=23)).strftime('%Y-%m-%d'),:], time_window)
            data_tw_analysis_minus_model = pd.Panel(pl.nan*pl.empty((2,len(data_tw_lastyear.major_axis),len(data_tw_lastyear.minor_axis))),\
                [pd.to_datetime(analysis_period[0]).strftime('%Y')+':'+(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)).strftime('%Y'),\
                (pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y')],\
                data_tw_lastyear.major_axis, data_tw_lastyear.minor_axis)
            data_tw_analysis_minus_model[pd.to_datetime(analysis_period[0]).strftime('%Y')+':'+(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)).strftime('%Y')] = data_tw_vsbefore['Analysis']-data_tw_vsbefore['Model']
            data_tw_analysis_minus_model[(pd.to_datetime(analysis_period[1])-pd.DateOffset(years=1)+pd.DateOffset(days=1)).strftime('%Y')] = \
                data_tw_lastyear['Analysis']-data_tw_lastyear['Model']
            data_tw_analysis_minus_model.values[pl.where(data_tw_analysis_minus_model==0.)] = pl.nan
            # Draw maps of diff only
	    if var in CBAR_MINMAX_DIFF_last_vs_before: 
		minim = CBAR_MINMAX_DIFF_last_vs_before[var][0]
		maxim = CBAR_MINMAX_DIFF_last_vs_before[var][1]
	    else: 
		minim = 0.
		maxim = 0. 
            #draw_map_vars(map_set, var+'_analysis_minus_model_lastyear_vs_before', data_tw_analysis_minus_model, graphics_dir_zone, time_window, None, \
            #                  fig_dpi, minim=minim, maxim=maxim, format_ticks='%.1e')
	    draw_map_analysis(map_set, data_tw_analysis_minus_model, data_tw_analysis_minus_model.items, graphics_dir_zone, \
                              var+'_analysis_minus_model_lastyear_vs_before',  time_window, None, \
                              fig_dpi,  minim=minim, maxim=maxim)
#
#
def postObs_SPOTVGT_vs_PROBAV(obs, data=None, minim=None, maxim=None):
    importLocals(currentframe())
    print('===================================================')
    print('Post-processing for observation: {0}'.format(obs)+(', model variable: {1}'.format(obs, OBS_MOD[obs]) if obs in OBS_MOD.keys() else ''))
    if data is None: 
        data = readAllData(obs, analysis_period, assim_hour, patch_frac, out_all_dir, post_from_pickle, mod_ana_dir, \
                          ['Model','Analysis','Obs'], IPNT=IPNT, patch=None, map_set=map_set)
    if 0 in data.shape:
        print('No dataset or no time step or no point. Continuing with next obs...')
        return
    #
    time_window = 'season'
    print('Statistics are computed for SPOT-VGT period (2007-2013) vs. PROBA-V period (2014-2016) for {0} at scale {1}'.format(obs,time_window))    
    scores_for_all_points_SPOTVGT = compute_scores_for_all_points(data.ix[:,(pd.to_datetime(analysis_period[0])).strftime('%Y-%m-%d'):pd.to_datetime('2013-12-31').strftime('%Y-%m-%d'),:], time_window)
    scores_for_all_points_PROBAV  = compute_scores_for_all_points(data.ix[:,pd.to_datetime('2015-01-01').strftime('%Y-%m-%d'):(pd.to_datetime(analysis_period[1])).strftime('%Y-%m-%d'),:], time_window)
    #
    draw_series_stats_SPOTVGT_PROBAV(scores_for_all_points_SPOTVGT, scores_for_all_points_PROBAV, time_window, obs, graphics_dir_zone, fig_dpi)
#
#
def postLST():
    importLocals(currentframe())
    print('===================================================')
    print('LST is going to be read for all times available (every 3 hours), it can take some time.')
    data = readAllData_all3H_LST(analysis_period, assim_hour, patch_frac, out_all_dir, \
                                 post_from_pickle, mod_ana_dir, ['Model','Analysis','Obs'])
    # Statistics for LST of all hours: only time series of bias per month for all hours available
    draw_series_bias_LST_all3H(data, graphics_dir_zone, fig_dpi)

def postObs_accuracy(obs, data=None):
    importLocals(currentframe())
    print('===================================================')
    print('Post-processing at monthly time scale for accuracy of observation {0}'.format(obs))
    if data is None: 
        data = readAllData(obs, analysis_period, assim_hour, patch_frac, out_all_dir, post_from_pickle, \
                           mod_ana_dir, ['Model','Analysis','Obs'], IPNT=IPNT,filter_nan=False)
    if 0 in data.shape:
        print('No dataset or no time step or no point. Continuing with next obs...')
        return
    #
    # Seperate statistics depending on the thresholds
    data_high = data.copy()
    data_low  = data.copy()
    #
    if   obs=='LAI'   or obs=='LAI_V1'  : threshold = 2.5 ; GCOS_accuracy_HighValues = 20 ; GCOS_accuracy_LowValues = 0.5
    elif obs=='FAPAR' or obs=='FAPAR_V1': threshold = 0.5 ; GCOS_accuracy_HighValues = 10 ; GCOS_accuracy_LowValues = 0.05 
    data_high['Model'   ].values[pl.where(data['Analysis'].values<=threshold)] = pl.nan
    data_high['Analysis'].values[pl.where(data['Analysis'].values<=threshold)] = pl.nan
    data_high['Obs'     ].values[pl.where(data['Analysis'].values<=threshold)] = pl.nan
    data_low[ 'Model'   ].values[pl.where(data['Analysis'].values> threshold)] = pl.nan
    data_low[ 'Analysis'].values[pl.where(data['Analysis'].values> threshold)] = pl.nan
    data_low[ 'Obs'     ].values[pl.where(data['Analysis'].values> threshold)] = pl.nan
    #
    # Resample data for relative RMSD computation
    time_window = 'month'
    data_high_tw = applyTimeWindow(data_high, time_window)
    data_low_tw  = applyTimeWindow(data_low , time_window)
    data_tw      = applyTimeWindow(data     , time_window)
    if 'Europe' in options_file: data_high_tw = data_high_tw.reindex(minor=pl.arange(map_set.X.size),copy=False) # if EU
    #
    # Compute scores
    print('Computing scores for obs {0}, can take some time'.format(obs))
    scores_high_per_point = compute_scores_per_point(data_high, time_window)
    scores_low_per_point  = compute_scores_per_point(data_low , time_window)
    if 'Europe' in options_file: 
        scores_high_per_point = scores_high_per_point.reindex(minor=pl.arange(map_set.X.size),copy=False) # if EU
        scores_low_per_point  = scores_low_per_point.reindex( minor=pl.arange(map_set.X.size),copy=False) # if EU
    #
    relRMSD_high_per_point = scores_high_per_point['RMSD']['Analysis'].values/data_high_tw['Analysis'].values
    #scores_high_per_point['RMSD']['Analysis'].values = scores_high_per_point['RMSD']['Analysis'].values/data_high_tw['Analysis'].values
    #
    # PLOT TIME SERIES
    #
    months    = MonthLocator(range(1, 13), bymonthday=1, interval=1)
    monthsFmt = DateFormatter("%b")
    #
    # High values
    pl.close()
    pl.figure(1,facecolor='w',figsize=(6,4))
    ax=pl.subplot(111)
    pl.plot(data_high_tw.major_axis.to_datetime(),pl.nanmean(relRMSD_high_per_point,axis=1)*100,'b-',linewidth=2)
    pl.fill_between(data_high_tw.major_axis.to_datetime(), pl.nanmean(relRMSD_high_per_point,axis=1)*100+pl.nanstd(relRMSD_high_per_point,axis=1)*100, \
                    pl.nanmean(relRMSD_high_per_point,axis=1)*100, facecolor='b', alpha=0.1, interpolate=True)
    pl.fill_between(data_high_tw.major_axis.to_datetime(), pl.nanmean(relRMSD_high_per_point,axis=1)*100-pl.nanstd(relRMSD_high_per_point,axis=1)*100, \
                    pl.nanmean(relRMSD_high_per_point,axis=1)*100, facecolor='b', alpha=0.1, interpolate=True)
    pl.plot(data_high_tw.major_axis.to_datetime(),pl.ones(data_high_tw.major_axis.shape)*GCOS_accuracy_HighValues,'r--',linewidth=2)
    if   obs=='LAI'   or obs=='LAI_V1'  : pl.ylim((0.,50.))
    elif obs=='FAPAR' or obs=='FAPAR_V1': pl.ylim((0.,30.))
    pl.ylabel('relative RMSD [%]',fontsize=14)
    pl.title('Relative RMSD = RMSD/analysis for {0}>{1}'.format(obs,threshold),fontsize=14)
    pl.grid()
    # Save figure
    fig_name = os.path.normpath('{0}/{1}_accuracyHighValues_{2}.png'.format(graphics_dir, obs, TW_USER_FIG[time_window],))
    pl.savefig(fig_name, dpi=150, bbox_inches='tight')
    print('Accuracy time series for high values of obs {0} saved in {1}'.format(obs,fig_name))
    #
    # Low values
    pl.close()
    pl.figure(1,facecolor='w',figsize=(6,4))
    ax=pl.subplot(111)
    pl.plot(scores_low_per_point.major_axis.to_datetime(),pl.nanmean(scores_low_per_point['RMSD']['Analysis'].values,axis=1),'b-',linewidth=2)
    pl.fill_between(scores_low_per_point.major_axis.to_datetime(), \
                    pl.nanmean(scores_low_per_point['RMSD']['Analysis'].values,axis=1)+pl.nanstd(scores_low_per_point['RMSD']['Analysis'].values,axis=1), \
                    pl.nanmean(scores_low_per_point['RMSD']['Analysis'].values,axis=1), facecolor='b', alpha=0.1, interpolate=True)
    pl.fill_between(scores_low_per_point.major_axis.to_datetime(), \
                    pl.nanmean(scores_low_per_point['RMSD']['Analysis'].values,axis=1)-pl.nanstd(scores_low_per_point['RMSD']['Analysis'].values,axis=1), \
                    pl.nanmean(scores_low_per_point['RMSD']['Analysis'].values,axis=1), facecolor='b', alpha=0.1, interpolate=True)
    pl.ylabel('RMSD [-]',fontsize=14)
    pl.plot(scores_low_per_point.major_axis.to_datetime(),pl.ones(scores_low_per_point.major_axis.shape)*GCOS_accuracy_LowValues,'r--',linewidth=2)
    if   obs=='LAI'   or obs=='LAI_V1'  : pl.ylim((0.,1.5)) ; pl.yticks(pl.arange(0.,1.6,0.5))
    elif obs=='FAPAR' or obs=='FAPAR_V1': pl.ylim((0.,0.25))
    pl.title('RMSD for {0}<={1}'.format(obs,threshold),fontsize=14)
    pl.grid()
    # Save figure
    fig_name = os.path.normpath('{0}/{1}_accuracyLowValues_{2}.png'.format(graphics_dir, obs, TW_USER_FIG[time_window],))
    pl.savefig(fig_name, dpi=150, bbox_inches='tight')
    print('Accuracy time series for low values of obs {0} saved in {1}'.format(obs,fig_name))
    #
    # PLOT MAPS of last year only
    #
    mod_lon, mod_lat = getModelGrid(mod_grid,mod_pgd_path)
    cmap_green_red = LinearSegmentedColormap.from_list("my_colormap", ((0 ,1.,0 ), (1.,0 ,0 )), N=2, gamma=1.0)
    if   'France' in options_file: m_FR = Basemap(projection='lcc', llcrnrlat=41.3,urcrnrlat=50.9,llcrnrlon=-4.1,urcrnrlon=11.0, lat_0=46.8, lon_0=2+(20+14.025/60)/60, lat_1=45+(53+56.108/60)/60, lat_2=47+(41+45.652/60)/60, resolution='i') ; m = m_FR
    elif 'Europe' in options_file: m_EU = Basemap(projection='cyl', llcrnrlat=25., urcrnrlat=75.5, llcrnrlon=-11.5, urcrnrlon=62.5, resolution='i') ; m = m_EU
    #
    # High values
    #
    pl.close()
    pl.figure(1,facecolor='w',figsize=(10,10))
    for i in range(12):
        pl.subplot(4,3,i+1)
        pl.title(pd.datetools.MONTHS[i])
        d=m.drawcoastlines()
        d=m.drawcountries()
        d=m.drawmapboundary()
        s=m.scatter(mod_lon,mod_lat,3,relRMSD_high_per_point[-12+i,:]*100.,edgecolor='none',vmin=0.,vmax=GCOS_accuracy_HighValues*2,cmap=cmap_green_red,latlon=True)
    #
    cbar=m.colorbar(s,location='bottom',pad="5%",spacing='proportional',format='%.0f',ticks=pl.linspace(0,GCOS_accuracy_HighValues*2,3),extend='max')
    cbar.ax.tick_params(labelsize=12)
    # Save figure
    fig_name = os.path.normpath('{0}/{1}_accuracyHighValues_MAP_{2}_{3}.png'.format(graphics_dir, obs, TW_USER_FIG[time_window],scores_low_per_point.major_axis[-1].year))
    pl.savefig(fig_name, dpi=150, bbox_inches='tight')
    print('Accuracy map for low values of obs {0} in {1} saved in {2}'.format(obs,scores_low_per_point.major_axis[-1].year,fig_name))
    #
    # Low values
    #
    pl.close()
    pl.figure(1,facecolor='w',figsize=(10,10))
    for i in range(12):
        pl.subplot(4,3,i+1)
        pl.title(pd.datetools.MONTHS[i])
        d=m.drawcoastlines()
        d=m.drawcountries()
        d=m.drawmapboundary()
        s=m.scatter(mod_lon,mod_lat,3,scores_low_per_point['RMSD']['Analysis'].values[-12+i,:],edgecolor='none',\
                    vmin=0.,vmax=GCOS_accuracy_LowValues*2, cmap=cmap_green_red,latlon=True) 
    #
    cbar=m.colorbar(s,location='bottom',pad="5%",spacing='proportional',format='%.2f',ticks=pl.linspace(0, GCOS_accuracy_LowValues*2,3),extend='max')
    cbar.ax.tick_params(labelsize=12)
    # Save figure
    fig_name = os.path.normpath('{0}/{1}_accuracyLowValues_MAP_{2}_{3}.png'.format(graphics_dir, obs, TW_USER_FIG[time_window],scores_low_per_point.major_axis[-1].year))
    pl.savefig(fig_name, dpi=150, bbox_inches='tight')
    print('Accuracy map for low values of obs {0} in {1} saved in {2}'.format(obs,scores_low_per_point.major_axis[-1].year,fig_name))
#
#
def postCopernicus():
    importLocals(currentframe())
    print('===================================================')
    print('## COPERNICUS ##')
    if not os.path.isfile(copernicus_fileForFigs): raise Exception('The file {0}, which lists the graphical files to be copied to {1}, is missing!'.format(copernicus_fileForFigs))
    copernicus_dir = os.path.normpath(graphics_dir_zone+'/copernicus/')+'/'
    if not os.path.isdir(copernicus_dir): os.makedirs(copernicus_dir)
    for graphic_info in reader(open(copernicus_fileForFigs)): # loop through each figure/table
        graphic_names = []
        for raw_reg_exp in graphic_info[1:]: # loop through the regular expression patterns of each figure/table
            f_matches = filter(lambda x: x is not None, [match(raw_reg_exp.strip(),f) for f in os.listdir(graphics_dir_zone)])
            graphic_names += [m.group(0) for m in f_matches] # graphical file names matching the regular expression pattern
        graphic_names.sort()
        num_sub = len(graphic_names) # number of sub-figures/tables
        sub_indices = [''] if num_sub == 1 else ascii_lowercase[:num_sub] # indices of sub-figures/tables
        for name, ix in zip(graphic_names,sub_indices):
            origin = os.path.normpath('{0}/{1}'.format(graphics_dir_zone,name))
            destination = os.path.normpath('{0}/{1}{2}_{3}'.format(copernicus_dir,graphic_info[0],ix,name))
            copyfile(origin,destination)
    print('Figures and tables for Copernicus have been copied to {0}\n'.format(copernicus_dir))


# ==============================================
# MAIN PROGRAM
# ==============================================

if __name__ == '__main__':
    # Check if the options file is passed correctly, read it, and check for missing options
    if len(argv) >= 2: options_file = getOptionsFile(argv)
    else: raise Exception('No option file given. Exiting')
    #
    # Initialization
    # from ldasPost_Copernicus import *
    # options_file = 'options_Europe_spin2007x4.py'
    # options_file = 'options_France_SAFRAN_newFORC_oldOPT.py'
    #
    initPost(options_file)
    #
    # Loop over zones
    #
    for zname,zone in post_zones.iteritems():
        # zname,zone='model zone',None
        if zname in ['Greenland','Oceans','Antarctic']: continue
        #
        print('\n*********************************************')
        print('  Postprocessing over {0}'.format(zname))
        print('*********************************************\n')
        #
        # Prepare zone selection
        map_set,graphics_dir_zone,IPNT = prepareZone(zname,zone,map_set,graphics_dir)
        #
        if 'last_year_vs_before' in extra_diagnostics.keys():
	    for obs in ['SWI', 'LAI_V1', 'FAPAR_V1', 'LST-6','LST-12']: #for obs in obs_assim+obs_NOassim: # but not SA ! ['SIF-L3']
                postObs_last_year_vs_before(obs)
	    for var in extra_out_vars: # + ['LAI']
                postExtraVar_last_year_vs_before(var)
	    if 'increments_assim' in extra_diagnostics.keys(): 
		postIncr_last_year_vs_before()
        #
        if 'SPOTVGT_vs_PROBAV' in extra_diagnostics.keys():
            for obs in extra_diagnostics['SPOTVGT_vs_PROBAV']: postObs_SPOTVGT_vs_PROBAV(obs)
        #
        if 'accuracy_obs_analysis' in extra_diagnostics.keys():
            for obs in extra_diagnostics['accuracy_obs_analysis']: postObs_accuracy(obs)
        #
        if 'LST_all3H' in extra_diagnostics.keys(): 
	    postLST()
        #
        # Save figures
        if copernicus_figs: postCopernicus()
        #
    print('Post-processing is complete. Thank you for watching.\n')

