# ----------------------------------------------
# IMPORT LIBRARIES
# ----------------------------------------------
# Python modules
#import pdb
import os
import pandas as pd
import pylab as pl
from re import match
# LDAS modules
from ldasModule import FILL, getObsRegions, h5File, Dataset, num2date

# ----------------------------------------------
# Datasets file format (CAN BE CHANGED)
# ----------------------------------------------

# Variable names in file (obs,qflags)
OBS_VAR = {'CGLS'       : {'LAI_V1'       :('LAI','LAI-QFLAG'),
                           'LAI_V2'       :('LAI','QFLAG'),#,'LENGTH_BEFORE','LENGTH_AFTER'),
                           'LAI300'       :('LAI','QFLAG'),#,'LENGTH_BEFORE','LENGTH_AFTER'),
                           #'LAI300'       :('LAI','RMSE','NOBS','LENGTH_BEFORE','LENGTH_AFTER','QFLAG'),
                           'SWI'          :('SWI/SWI_001','QFLAG/QFLAG_001','FREEZE_THAW/SSF'),
                           'SA'           :('AL-BH-BB','AL-BH-QFLAG'),
                           'LST'          :('LST','Q_FLAGS'), # Not fully implemented (from REGIONS to GLOBAL)
                           'FAPAR_V1'     :('FAPAR','FAPAR-QFLAG'),
                           'FCOVER_V1'    :('FCOVER','FCOVER-QFLAG')},
           'CGLS_12D'   : {'LAI_V1'       :('LAI','LAI-QFLAG')},
           'CGLS_20D'   : {'LAI_V1'       :('LAI',),
                           'LAI_MC'       :('LAI',),
                           'LAI_agg'      :('LAI_agg',)},
           'ESACCI'     : {'SSM'          :('sm',),
                           'SSM_COMBINED' :('sm',),
                           'SSM_ACTIVE'   :('sm',),
                           'SSM_PASSIVE'  :('sm',)},
           'GOME2'      : {'SIF'          :('SIF',)},
           'GOME-F'     : {'SIF-F'        :('SIF_740',),
                           'SIF-L3'       :('SIF_740','SIF_740_std','Counts'),
                           'SIF-L3_V27'   :('SIF','SIF_std','Counts'),
                           'SIF-L3_V27_d' :('SIF_daily_average','SIF_std','Counts'),
                           'SIF-L3_V27_p' :('Par_normalized_SIF','Par_normalized_SIF_std','Counts')},
           'GLEAM'      : {'E_ACTUAL'     :('E',),
                           'E_i'          :('Ei',),
                           'E_t'          :('Et',),
                           'E_b'          :('Eb',)},
           'FLUXNET_MTE': {'FLGPP'        :('gpp',),
                           'FLLE'         :('EnsembleLEcor_May12',)},
           'FLUXCOM'    : {'FLCGPP'       :('GPP',)},
           'TUWIEN'     : {'SM_QD'        :('sm',)},
          }


# ----------------------------------------------
# Dataset specific functions (CAN BE CHANGED)
# ----------------------------------------------

def updateObsDates(obs_root_dir,obs_dataset,obs_var):
    '''
    Function to update the DATASET_VAR_DATES.csv files
    Arguments :
        obs_root_dir (str): root directory of observations
        obs_dataset (str): observation dataset
        obs_var (str): observation variable name
    '''
    if obs_root_dir is None: obs_root_dir = '/cnrm/vegeo/LDAS/OBSERVATIONS/'
    # 1. Get available dates and file names for the first region
    # ----------------------------------------------------------------------------------
    obs_daily = True
    regions = getObsRegions(obs_root_dir,obs_dataset,obs_var)
    obs_region = regions.keys()[0]
    obs_data_dir = os.path.normpath('/'.join((obs_root_dir,obs_dataset,obs_var,obs_region)))
    files_name = []
    #
    # CGLS DATASET
    if obs_dataset == 'CGLS':
        if obs_var in ['LAI_V1','SWI','FAPAR_V1','SA','FCOVER_V1']:
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in os.listdir(obs_data_dir) if match('\d{8}',d) is not None]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d')
                tmp = [f for f in os.listdir(obs_data_dir+'/'+date_str) if match('g2_BIOPAR_.*h5$',f) is not None][0]
                files_name.append(os.path.join(date_str,tmp).replace(obs_region,'XXXREGIONXXX'))
        elif obs_var in ['LAI_V2','LAI300']:
            r_e = 'c_gls_.*_(\d{8})0000.+'
            assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d') for f in os.listdir(obs_data_dir)]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                tmp = [f for f in os.listdir(obs_data_dir) if match('c_gls_.*_{0}00_.*nc'.format(date_str),f) is not None][0]
                files_name.append(tmp)
            # Move dates from end to middle of decade
            #assim_dates = [pd.datetime(d.year,d.month,(d.day-5)/10*10+5) for d in assim_dates]
        elif obs_var == 'LST':
            obs_daily = False
            r_e = 'g2_BIOPAR_LST_(\d{10})00.+'
            assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d%H') for g in os.walk(obs_data_dir) for f in g[2]]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                tmp_dir = '{1}/{2}/{3}'.format(obs_data_dir,date_str[:4],date_str[4:6],date_str[6:8])
                tmp = [f for f in os.listdir(obs_data_dir+'/'+tmp_dir) if match('g2_BIOPAR_.*_{0}00_.*nc$'.format(date_str),f) is not None][0]
                files_name.append(os.path.join(tmp_dir,tmp).replace(obs_region,'XXXREGIONXXX'))
    #
    # CGLS_12D DATASET
    elif obs_dataset == 'CGLS_12D':
        if obs_var == 'LAI_V1':
            r_e = 'g2_BIOPAR_LAI_(\d{8})0000.+'
            assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d') for f in os.listdir(obs_data_dir)]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                tmp = [f for f in os.listdir(obs_data_dir) if match('g2_BIOPAR_.*_{0}00_.*h5'.format(date_str),f) is not None][0]
                files_name.append(tmp.replace(obs_region,'XXXREGIONXXX'))
    #
    # CGLS_20D DATASET
    elif obs_dataset == 'CGLS_20D':
        if obs_var == 'LAI_V1':
            r_e = 'g2_BIOPAR_LAI_(\d{8})0000.+'
            assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d') for f in os.listdir(obs_data_dir)]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                tmp = [f for f in os.listdir(obs_data_dir) if match('g2_BIOPAR_.*_{0}00_.*nc'.format(date_str),f) is not None][0]
                files_name.append(tmp.replace(obs_region,'XXXREGIONXXX'))
        if obs_var == 'LAI_V2':
            r_e = 'c_gls_.*_(\d{8})0000.+'
            assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d') for f in os.listdir(obs_data_dir)]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                tmp = [f for f in os.listdir(obs_data_dir) if match('c_gls_.*_{0}00_.*nc'.format(date_str),f) is not None][0]
                files_name.append(tmp)
    #
    # ESACCI DATASET
    elif obs_dataset == 'ESACCI':
        if obs_var == 'SSM':
            r_e = 'ESACCI-SOILMOISTURE-L3S-SSMV-COMBINED-(\d{8})000000-fv02.2.nc'
            date_str = []
            for d in os.listdir(obs_data_dir):
                if not os.path.isdir(obs_data_dir+'/'+d): continue
                date_str = date_str+[match(r_e,f).groups(1)[0] for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None]
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                files_name.append('{0}/ESACCI-SOILMOISTURE-L3S-SSMV-COMBINED-{1}0000-fv02.2.nc'.format(date_str[:4],date_str))
        if obs_var in ['SSM_COMBINED','SSM_ACTIVE','SSM_PASSIVE']:
            if   obs_var == 'SSM_COMBINED': new_var = 'SSMV-COMBINED'
            elif obs_var == 'SSM_PASSIVE' : new_var = 'SSMV-PASSIVE'
            elif obs_var == 'SSM_ACTIVE'  : new_var = 'SSMS-ACTIVE'
            r_e = 'ESACCI-SOILMOISTURE-L3S-'+new_var+'-(\d{8})000000-fv03.1.nc'
            date_str = []
            for d in os.listdir(obs_data_dir):
                if not os.path.isdir(obs_data_dir+'/'+d): continue
                date_str = date_str+[match(r_e,f).groups(1)[0] for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None]
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d%H')
                files_name.append(('{0}/ESACCI-SOILMOISTURE-L3S-'+new_var+'-{1}0000-fv03.1.nc').format(date_str[:4],date_str))
    #
    # GOME2 DATASET
    elif obs_dataset == 'GOME2':
        if obs_var == 'SIF':
            r_e = 'GOME-2_SIF_(\d{8}).nc'
            date_str = []
            for d in os.listdir(obs_data_dir):
                if not os.path.isdir(obs_data_dir+'/'+d): continue
                date_str = date_str+[match(r_e,f).groups(1)[0] for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None]
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d')
                files_name.append('{0}/GOME-2_SIF_{1}.nc'.format(date_str[:4],date_str))
    #
    # GOME-F DATASET
    elif obs_dataset == 'GOME-F':
        if obs_var == 'SIF-F':
            r_e = 'SIF-F_interp_MetopA_V26_L2_(\d{8}).nc'
            date_str = []
            for d in os.listdir(obs_data_dir):
                if not os.path.isdir(obs_data_dir+'/'+d): continue
                date_str = date_str+[match(r_e,f).groups(1)[0] for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None]
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d')
                files_name.append('{0}/SIF-F_interp_MetopA_V26_L2_{1}.nc'.format(date_str[:4],date_str))
        if obs_var == 'SIF-L3':
            r_e = 'ret_f_nr5_nsvd12_v26_waves734_nolog.grid_SIF__v20_1x1_(\d{8})_31.nc'
            date_str = []
            for d in os.listdir(obs_data_dir):
                if not os.path.isdir(obs_data_dir+'/'+d): continue
                date_str = date_str+[match(r_e,f).groups(1)[0][:-2]+'15' for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None][:-2]
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d')
                files_name.append('{0}/ret_f_nr5_nsvd12_v26_waves734_nolog.grid_SIF__v20_1x1_{1}01_31.nc'.format(date_str[:4],date_str[:6]))
        if obs_var in ['SIF-L3_V27','SIF-L3_V27_d','SIF-L3_V27_p'] :
            r_e = 'ret_f_nr5_nsvd12_v26_waves734_nolog.grid_SIF_v27_(\d{8})_31.nc'
            date_str = []
            for d in os.listdir(obs_data_dir):
                if not os.path.isdir(obs_data_dir+'/'+d): continue
                date_str = date_str+[match(r_e,f).groups(1)[0][:-2]+'15' for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None][:-2]
            assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
            for curr_date in assim_dates:
                date_str = curr_date.strftime('%Y%m%d')
                files_name.append('{0}/ret_f_nr5_nsvd12_v26_waves734_nolog.grid_SIF_v27_{1}01_31.nc'.format(date_str[:4],date_str[:6]))
    #
    # GLEAM
    elif obs_dataset == 'GLEAM':
        if obs_var == 'E_ACTUAL':
            filename = '{0}/{1}/{2}/{3}/E_GLEAM_v3.1a_1980-2016.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['E_GLEAM_v3.1a_1980-2016.nc']*len(assim_dates)
        elif obs_var == 'E_t':
            filename = '{0}/{1}/{2}/{3}/Et_GLEAM_v3.1_1980-2016.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['Et_GLEAM_v3.1a_1980-2016.nc']*len(assim_dates)
        elif obs_var == 'E_i':
            filename = '{0}/{1}/{2}/{3}/Ei_GLEAM_v3.1a_1980-2016.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['Ei_GLEAM_v3.1a_1980-2016.nc']*len(assim_dates)
        elif obs_var == 'E_b':
            filename = '{0}/{1}/{2}/{3}/Eb_GLEAM_v3.1a_1980-2016.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['Eb_GLEAM_v3.1a_1980-2016.nc']*len(assim_dates)
    #
    # FLUXNET_MTE
    elif obs_dataset == 'FLUXNET_MTE':
        if obs_var == 'FLGPP':
            filename = '{0}/{1}/{2}/{3}/EnsembleGPP_GL.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['EnsembleGPP_GL.nc']*len(assim_dates)
        elif obs_var == 'FLLE':
            filename = '{0}/{1}/{2}/{3}/EnsembleLEcor_May12.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['EnsembleLEcor_May12.nc']*len(assim_dates)
    #
    # FLUXCOM
    elif obs_dataset == 'FLUXCOM':
        if obs_var == 'FLCGPP':
            assim_dates,files_name = pl.array([]),[]
            files = os.listdir(obs_data_dir)
            files.sort()
            for f in files:
                if not f[:24]+f[28:] == 'CRUNCEPv6.ANN.GPP.daily..nc': continue
                with Dataset(obs_data_dir+'/'+f) as nc:
                    dat = num2date(nc.variables['time'][:],nc.variables['time'].units)
                assim_dates = pl.hstack((assim_dates,dat))
                files_name = files_name+len(dat)*[f]
    #
    # TUWIEN
    elif obs_dataset == 'TUWIEN':
        if obs_var == 'SM_QD':
            filename = '{0}/{1}/{2}/{3}/HSAF_SM_QD_{3}.nc'.format(obs_root_dir,obs_dataset,obs_var,obs_region)
            nc = Dataset(filename)
            assim_dates = num2date(nc.variables['time'][:],nc.variables['time'].units)
            files_name = ['HSAF_SM_QD_XXXREGIONXXX.nc']*len(assim_dates)
    #
    # 2. Check if dates are the same for the other regions
    # ----------------------------------------------------------------------------------
    I_remove = []
    for i,f in zip(range(len(files_name)),files_name): #d,f in zip(assim_dates,files_name):
        for r in [r for r in regions if not r==obs_region]:
            obs_data_dir = os.path.normpath('/'.join((obs_root_dir,obs_dataset,obs_var,r)))
            if not os.path.isfile(os.path.join(obs_data_dir,f).replace('XXXREGIONXXX',r)):
                print('Date {0} is available for {1} but not for {2}. It has been removed'\
                    .format(assim_dates[i].strftime('%Y%m%d%H'),obs_region,r))
                print(os.path.join(obs_data_dir,f).replace('XXXREGIONXXX',r))
                I_remove.append(i)
                break
    assim_dates = pl.delete(assim_dates,I_remove)
    files_name = pl.delete(files_name,I_remove)
    #
    # 3. Write the date file
    # ----------------------------------------------------------------------------------
    var_date_file = os.path.join(obs_root_dir,obs_dataset,obs_var,obs_dataset+'_'+obs_var+'_DATES.csv')
    with open(var_date_file,'w') as fid:
        for d,f in zip(assim_dates,files_name):
            dat = d.strftime('%Y%m%d%H')
            if obs_daily: dat = dat[:-2]+'XX'
            fid.write('{0},{1}\n'.format(dat,f))


def flagScreen(obs_dataset,obs_var,obs_values,obs_flags,filenameH5=None):
    '''
    Screen observation data using quality flags for one daily time step (Fortran90 implementation).
    Also used to rescale observation when not done properly in readFromFile (e.g. non conventional scale factors, latitude order)
    Arguments:
        obs_dataset (str): observation dataset
        obs_var (str): observation variable name.
        obs_values (numpy.ndarray): observation values for one daily time step.
        obs_flags (list): quality flags for one daily time step
    Returns:
        obs_values (numpy.ndarray): screened observation values for one daily time step.
    '''
    if obs_dataset in ['CGLS','CGLS_12D']:
        if obs_var == 'LAI_V1':
            obs_values[obs_values==0] = FILL # remove LAI when null
            obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[1]+range(5,16),[],65530,65530,1./(30*30))
        elif obs_var == 'LAI_V2':
            obs_values[obs_values==0] = FILL # remove LAI when null
            obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[0,2,5,6,7,8],[],7,65530,1.)
        elif obs_var == 'LAI300':
            obs_values[obs_values==0] = FILL # remove LAI when null
            flag_bits = [] # 0:sea/land, 1:EBF class, 2:interpolate, 4:EBF obs/previous, 5-6:kind of interpolation
            obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],flag_bits,[],7,255,1.)
        elif obs_var == 'SWI':
            obs_values[obs_values!=FILL] /= 100 # convert from % to decimal
            obs_values[obs_flags[0]<80] = FILL # Remove observations when not enough SSM to calculate SWI (cf ATBD)
            obs_values[pl.logical_and(obs_flags[1]>=2,obs_flags[1]!=FILL)] = FILL # Remove frozen observations
        elif obs_var == 'FAPAR_V1':
            if filenameH5 is None: raise Exception('Needs filename of H5 file to get the scaling factor for FAPAR')
            fid = h5File(filenameH5,'r')
            scaling_factor = float(fid.get('FAPAR').attrs.get('SCALING_FACTOR'))
            fid.close()
            #obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[0,1,5,7],[2,3,6]+range(9,13),23500,4,1./(100*250))
            obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[0,1,5,7],[2,3,6]+range(9,13),0.94*(scaling_factor*250),4,1./(scaling_factor*250))
        elif obs_var == 'SA':
            obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[0,2,5,6],[3,7,8,9,10,11],65530*1e4,2,1./(1e4*1e4))
        elif obs_var == 'LST':
            #obs_values[obs_values==-8000*100] = FILL
            #obs_values[obs_values!=FILL] /= (100*100) # Problem with scaling factor in readFromFile (* instead of /, so need to double /)
            #obs_values[obs_values!=FILL] += 273.15    
            obs_values[not obs_flags[0] in pl.arange(4,8)] = FILL # 4,5,6,7 values are cloud-free pixels for the 4 satellites (table 4 p.50 of ATBD)
            #obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[4,5,6,7],[],65530,10,1)
        elif obs_var == 'FCOVER_V1':
            obs_values = _checkFlagBitsRescale(obs_values,obs_flags[0],[0,1]+range(5,16),[],65530,65530,1./(250*250))
    #elif obs_dataset == 'ESACCI':# qflag already applied on obs_values for ESA-CCI SSM
    #    if obs_var == 'SSM':
    elif obs_dataset == 'GOME2':
        if obs_var == 'SIF':
            obs_values[obs_values<0] = FILL
    elif obs_dataset == 'GOME-F':
        if obs_var == 'SIF-L3':
            obs_values[obs_values>6.]   = FILL
            obs_values[obs_flags[0]>2.] = FILL # SIF_740_std>2 => discard
            obs_values[obs_flags[1]<5]  = FILL # Counts<5 => discard
        elif obs_var in ['SIF-L3_V27','SIF-L3_V27_d','SIF-L3_V27_p']:
            #obs_values[obs_values<0.]   = FILL
            obs_values[obs_values>6.]   = FILL
            obs_values[obs_flags[0]>2.] = FILL # SIF_std>2 => discard
            obs_values[obs_flags[1]<5]  = FILL # Counts<5 => discard
    elif obs_dataset == 'FLUXNET_MTE':
        if obs_var == 'FLGPP':
            obs_values[pl.where(obs_values != FILL)] = obs_values[pl.where(obs_values != FILL)] *3600.*24.*1000. # convert from kg/m2/s to g/m2/day
    return obs_values


# ----------------------------------------------
# Private functions
# ----------------------------------------------

def _checkFlagBitsRescale(obs_values,flag_data,must_bits,opt_bits,obs_thre,score_thre,scale_fact):
    '''
    Flag-screen and re-scale observation data (Python implementation).
    Arguments:
        obs_values (numpy.ndarray): observation values for one daily time step.
        flag_data (numpy.ndarray): quality flags for one daily time step.
        must_bits (list): positions of quality flag bits that must be satisfactory.
        opt_bits (list): positions of optional quality flag bits.
        obs_thre (int): threshold above which observations are discarded.
        score_thre (int): minimum number of satisfactory flag bits.
        scale_fact (float): observation scale factor.
    Returns:
        obs_values (numpy.ndarray): observation values for one daily time step.
    '''
    flag_data = flag_data.astype('uint')
    obs_values[obs_values==FILL] = pl.nan
    for pos in must_bits: # compulsory bits
        obs_values[_bitget(flag_data,pos)] = pl.nan
    flag_scores = pl.zeros(obs_values.shape,pl.uint8)
    for pos in opt_bits: # optional bits
        flag_scores += _bitget(flag_data,pos)
    obs_values[pl.logical_or(flag_scores > score_thre, obs_values > obs_thre)] = pl.nan
    obs_values *= scale_fact
    obs_values[pl.isnan(obs_values)] = FILL
    return obs_values

def _bitget(value, bit_number):
    '''
    Get bit at \'bit_number\' position in \'value\'.
    '''
    return (value & (1 << bit_number)) != 0

