
exp_name = 'Copernicus_Europe_spin2007x4' # Name of experiment

# -------------------------------
# ASSIMILATION
# -------------------------------
analysis_period = ['2007-01-01', '2016-12-31'] # Analysis period.
assim_hour = 9 # Assimilation hour (in range(24)).

# -------------------------------
# MODEL
# -------------------------------
# Grid
mod_grid  = {'type': 'uniform', 'res': 0.5, 'zone': [[-11.5,75.5],[62.5,25.]],'proj':None} # Model grid type ([[ul_lon,ul_lat],[lr_lon,lr_lat]])
#trip_grid = {'type': 'uniform', 'res': 0.5, 'zone': [[-180,90],[180,-90]]}
# Output format
mod_out_format = {'format':'netcdf'} # Model output file format.

# -------------------------------
# FORCING
# -------------------------------
get_raw_forcing    =  True           # Retrieve raw forcing files (True) or not (False)
freq_forcing       =  3              # Time frequency of the forcing data in hours
nc_version         = 'NETCDF3_64BIT' # NetCDF version; possbile values: 'NETCDF4', 'NETCDF4_CLASSIC', 'NETCDF3_64BIT', 'NETCDF3_CLASSIC'
raw_forcing_format = 'SFX_NC'        # files already in SFX format, 1 file per month
FORC_VARS  = {'Tair':'Tair','Qair':'Qair','Wind':'Wind','DIR_SWdown':'DIR_SWdown','SCA_SWdown':'SCA_SWdown','LWdown':'LWdown','PSurf':'PSurf','Rainf':'Rainf','Snowf':'Snowf','CO2air':'CO2air','Wind_DIR':'Wind_DIR'}
#raw_forcing_format = 'VAR_NC' # seperated files for each variable (1 directory per variable), monthly files
#FORC_VARS  = {'Tair':'Tair' ,'Qair':'Qair' ,'Wind':'Wind' ,'DIR_SWdown':'SWdown' ,'LWdown':'LWdown' ,'Rainf':'Rainf'     ,'Snowf':'Snowf'     ,'PSurf':'PSurf'}

# -------------------------------
# OBSERVATIONS
# -------------------------------
# Variables to be assimilated (names and folders)GLEAM'      : {'E_ACTUAL'     :('E',),
obs_names                   = ['SWI' ,'LAI_V1','FAPAR_V1','SA'   ,'LST'  ,'SIF-L3_V27_d','E_ACTUAL'] # List of observation variables to be pre-processed
obs_datasets                = ['CGLS','CGLS'  ,'CGLS'    ,'CGLS' ,'CGLS' ,'GOME-F'      ,'GLEAM'   ] # Version of each observation variable
to_assim                    = [ True , True   , False    , False , False , False        , False    ] # Booleans indicating which observation variables (listed in obs_names) are assimilated in the analysis
obs_from_pickle             = [ True  ,True   , True     , True  , True  , True         , True     ]
# Grids
obs_grids                   = ['def'  ,'def'  , 'def'    , 'def' , 'def' , 'def'        ,'def'     ] # Observation grid types
grid_transform_methods      = ['def'  ,'def'  , 'def'    , 'def' , 'def' ,'nearest'     ,'def'     ] # Methods used to interpolate observation data on the model grid
grid_transform_parameters   = ['def'  ,'def'  , 'def'    , 'def' , 'def' , 'def'        ,'def'     ] # Parameters for interpolating observation data on the model grid
# Screening with land surface properties
surf_screen_features        = [['urban_fraction', 'altitude'], [] , [] , [] , [] , []   , []       ] # Surface features used to screen observations (1 list per variable)
surf_screen_thresholds      = [[.15, 1500]                   , [] , [] , [] , [] , []   , []       ] # Thresholds of surface features (listed in surf_screen_features) for discarding observations
# Screening with model output variables
screen_mod_names            = [['WGI1'], []   , []       , []    , []    , []           , []       ] # Model output variables used to screen observations (1 list per variable; the empty list [] means no screening)
screen_mod_steps            = [[0]    , []    , []       , []    , []    , []           , []       ] # Time steps (in range(mod_out_nsteps)) of model output used for screening
screen_mod_thres            = [[0]    , []    , []       , []    , []    , []           , []       ] # Thresholds of model output variables (listed in screen_mod_names) for discarding observations 
# Re-scaling using model output variables
rescale_methods             = ['cdf_matching',None, None , None  , None  , None         , None     ] # Methods for re-scaling observations using model output (None means no re-scaling)
rescale_mod_names           = ['WG1' , None   , None     , None  , None  , None         , None     ] # Model output variables used for re-scaling observations
rescale_mod_steps           = [0     , None   , None     , None  , None  , None         , None     ] # Time steps (in range(mod_out_nsteps)) of model output used for re-scaling
rescale_max_miss            = [.9    , None   , None     , None  , None  , None         , None     ] # Maximum fraction of missing observations for re-scaling a grid-point time series; if not satisfied, observations are set to missing
rescale_calibs              = [True  , None   , None     , None  , None  , None         , None     ] # Booleans indicating whether the re-scaling parameters should be calibrated
rescale_calib_periods       = [['2007-01-01','2014-12-31'],[],[],[], []  , []           , []       ] # Periods (start and end year, both included) over which the re-scaling parameters are calibrated

# -------------------------------
# DIRECTORIES
# -------------------------------
# Input
mod_pgd_path    = '/cnrm/vegeo/leroux/NO_SAVE/SFX_simulations/Copernicus_Europe/pgd_prep/' # Must include PGD.nc, PGD_fractown.nc and PREP.nc
raw_forcing_dir = '/cnrm/est/DATA/NO_SAVE/FORCAGES/SFX_ERAInterim/0.5deg/3hourly/' # Folder from where the raw forcing data is read
#raw_forcing_dir = '/cnrm/vdr/DATA/NO_SAVE/FORCAGES/E2O/0.5deg/3hourly/' # Folder from where the raw forcing data is read
obs_root_dir    = '/cnrm/vegeo/LDAS/OBSERVATIONS/' # Root folder of all observation files
openloop_dir    = '/cnrm/vegeo/leroux/NO_SAVE/SFX_simulations/Copernicus_Europe/ol_2007-2016_spin2007x4/' # Folder from where openloop output is read in pre- and post-processing steps
analysis_dir    = '/cnrm/vegeo/leroux/NO_SAVE/SFX_simulations/Copernicus_Europe/ekf_2007-2016_spin2007x4_cdfNEW/' # Folder from where analysis output is read in post-processing step
# Output
forcing_dir   = '/cnrm/vegeo/leroux/LDAS/LDAS_v1.2.2/forcing/'+exp_name+'/' # Folder where NetCDF (as well as intermediate binary) forcing files are written.
out_obs_dir   = '/cnrm/vegeo/leroux/LDAS/LDAS_v1.2.2/observations/'+exp_name+'/' # Folder where processed (CANARI files and PData) observations are written
out_mod_dir   = '/cnrm/vegeo/leroux/LDAS/LDAS_v1.2.2/results/'+exp_name+'/ol/' # Folder where PData (when asked) model output data are written
out_ana_dir   = '/cnrm/vegeo/leroux/LDAS/LDAS_v1.2.2/results/'+exp_name+'/ekf/' # Folder where PData (when asked) analysis output data are written
graphics_dir  = '/cnrm/vegeo/leroux/LDAS/LDAS_v1.2.2/results/'+exp_name+'/figures/' # Folder from where graphics files are written in the post-processing step.

# -------------------------------
# POST-PROCESSING
# -------------------------------
post_mod_from_pickle   = True # if True: try to read pickle files; if False or no pickle files: read model output files and save to pickle files.
post_ana_from_pickle   = True
post_obs_from_pickle   = True
post_zones             = {'def': None} # Default
#post_zones             = {'def': None, 'all': 'STN_Continents_1D.nc'}
#post_zones             = {'all': 'STN_Continents_1D.nc'}
extra_out_vars         = ['WG2','WG3','WGTOT','EVAPC','DRAINC','RC_ECO','RUNOFFC','GPPC','NEEC'] # 'LAI' (but will overwrite figure names...)
trip_vars              = [] #['QDIS']
list_time_windows      = ['day' , 'month' , 'season' , 'year' , 'all']
var_spatial_maps_out   = [False , True    , False    , False  , True ] # [False , True    , False    , False  , True ]
var_time_series_out    = [True  , True    , True     , True   , False] # [True  , True    , True     , True   , False]
stats_spatial_maps_out = [False , False   , True     , True   , True ] # [False , False   , True     , True   , True ]
stats_time_series_out  = [False , False   , True     , True   , False] # [False , False   , True     , True   , False]
stats_table_out        = [False , False   , True     , True   , True ] # [False , False   , True     , True   , True ]
'''
list_time_windows      = ['day' , 'month' , 'season' , 'year' , 'all']
var_spatial_maps_out   = [False , True    , False    , False  , True ] # [False , True    , False    , False  , True ]
var_time_series_out    = [True  , True    , True     , True   , False] # [True  , True    , True     , True   , False]
stats_spatial_maps_out = [False , True    , True     , True   , True ] # [False , False   , True     , True   , True ]
stats_time_series_out  = [False , True    , True     , True   , False] # [False , False   , True     , True   , False]
stats_table_out        = [False , True    , True     , True   , True ] # [False , False   , True     , True   , True ]
'''
extra_diagnostics      = {'increments_assim'     :['ANAL_INCR1'    ,'ANAL_INCR2'    ],\
                          'increments_names'     :['LAI increments','WG2 increments'],\
                          'increments_limits'    :[[-0.15,0.15]    ,[-1e-3,1e-3]],\
                          'increments_lim_vs'    :[[-2e-2,2e-2]    ,[-1e-3,1e-3]    ],\
                          'jacobian_assim'       :['HO_1_1'   ,'HO_1_2'   ,'HO_2_1'   ,'HO_2_2'   ],\
                          'jacobian_names'       :['dWG1/dLAI','dWG1/dWG2','dLAI/dLAI','dLAI/dWG2'],\
			  'last_year_vs_before'  :'',\
                          'SPOTVGT_vs_PROBAV'    :['LAI_V1','FAPAR_V1'],\
			  'LST_all3H'            :'',\
                          'accuracy_obs_analysis':['LAI_V1','FAPAR_V1'],\
                         }
#			  'patch'                :['LAI_V1','SWI'],\
#                         'major_patch'          :['LAI','SWI','FAPAR','SIF-F'],\
patch_out              = [4,5,6,7,8,10] # patch number between 1 and 12
modexp_names           = 'def' # if 'def': ['Model','Analysis']
fig_dpi                = 300 # Resolution for saving raster figures.
# COPERNICUS figures to save to put in the final report
copernicus_figs        = True
copernicus_fileForFigs = 'copernicus_figures_Europe.csv'

