
exp_name = 'sfx-trip' # Name of experiment

# -------------------------------
# ASSIMILATION
# -------------------------------
analysis_period = ['2000-01-01', '2014-12-31'] # Analysis period.
assim_hour = 9 # Assimilation hour (in range(24)).

# -------------------------------
# MODEL
# -------------------------------
# Grid
mod_grid = {'type': 'uniform', 'res': 1., 'zone': [[-180,90],[180,-90]]} # Model grid type ([[ul_lon,ul_lat],[lr_lon,lr_lat]])
# Output format
mod_out_format = {'format':'netcdf', 'n_steps':2} # Model output file format.

# -------------------------------
# FORCING
# -------------------------------
get_raw_forcing = False # Retrieve raw forcing files (True) or not (False)
freq_forcing = 3 # Time frequency of the forcing data in hours
nc_version = 'NETCDF3_64BIT' # NetCDF version; possbile values: 'NETCDF4', 'NETCDF4_CLASSIC', 'NETCDF3_64BIT', 'NETCDF3_CLASSIC'
raw_forcing_format = 'SFX_NC' # files already in SFX format, 1 file per month
FORC_VARS  = {'Tair':'Tair','Qair':'Qair','Wind':'Wind','DIR_SWdown':'DIR_SWdown','SCA_SWdown':'SCA_SWdown','LWdown':'LWdown','PSurf':'PSurf','Rainf':'Rainf','Snowf':'Snowf','CO2air':'CO2air','Wind_DIR':'Wind_DIR'}

# -------------------------------
# OBSERVATIONS
# -------------------------------
# Variables to be assimilated (names and folders)
obs_names                   = ['SSM'                            , 'LAI'         ] # List of observation variables to be pre-processed.
obs_datasets                = ['ESACCI'                         , 'BIOPAR_12D'  ] # Version of each observation variable
to_assim                    = [True                             , True          ] # Booleans indicating which observation variables (listed in obs_names) are assimilated in the analysis.
obs_from_pickle             = [True                             , True          ]
# Grids
obs_grids                   = ['def'                            , 'def'         ] # Observation grid types.
grid_transform_methods      = ['def'                            , 'def'         ] # Methods used to interpolate observation data on the model grid.
grid_transform_parameters   = ['def'                            , 'def'         ] # Parameters for interpolating observation data on the model grid.
# Screening with land surface properties
surf_screen_features        = [['urban_fraction', 'altitude']   , []            ] # Surface features used to screen observations (1 list per variable).
surf_screen_thresholds      = [[.15, 1500]                      , []            ] # Thresholds of surface features (listed in surf_screen_features) for discarding observations.
# Screening with model output variables
screen_mod_names            = [['WGI2']                         , []            ] # Model output variables used to screen observations (1 list per variable; the empty list [] means no screening).
screen_mod_steps            = [[0]                              , []            ] # Time steps (in range(mod_out_nsteps)) of model output used for screening.
screen_mod_thres            = [[0]                              , []            ] # Thresholds of model output variables (listed in screen_mod_names) for discarding observations.
# Re-scaling using model output variables
rescale_methods             = ['cdf_matching'                   , None          ] # Methods for re-scaling observations using model output (None means no re-scaling).
rescale_mod_names           = ['WG2'                            , None          ] # Model output variables used for re-scaling observations.
rescale_mod_steps           = [0                                , None          ] # Time steps (in range(mod_out_nsteps)) of model output used for re-scaling.
rescale_max_miss            = [.9                               , None          ] # Maximum fraction of missing observations for re-scaling a grid-point time series; if not satisfied, observations are set to missing.
rescale_calibs              = [True                             , None          ] # Booleans indicating whether the re-scaling parameters should be calibrated.
rescale_calib_periods       = [['2000-01-01', '2014-12-31']     , []            ] # Periods (start and end year, both included) over which the re-scaling parameters are calibrated.

# -------------------------------
# DIRECTORIES
# -------------------------------
# Input
mod_pgd_path = '/home/muniers/LDAS/GLO_1D/sfx-trip/pgd_prep/' # Must include PGD.nc, PGD_fractown.nc and PREP.nc
raw_forcing_dir = '/cnrm/vegeo/munier/e2o/GLO_025D/forcings/monthly/' # Folder from where the raw forcing data is read
obs_root_dir = '/cnrm/vegeo/LDAS/OBSERVATIONS/' # Root folder of all observation files
openloop_dir = '/cnrm/vegeo/munier/e2o/GLO_1D/results/LAImin/eco/' # Folder from where openloop output is read in pre- and post-processing steps
analysis_dir = '/cnrm/vegeo/munier/e2o/GLO_1D/results/LAImin/lai/' # Folder from where analysis output is read in post-processing step
# Output
forcing_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/forcings/daily/' # Folder where NetCDF (as well as intermediate binary) forcing files are written.
out_obs_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/observations/sfx-trip/' # Folder where processed (CANARI files) observations are written
out_mod_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/results/eco_lai/eco/' # Folder where PData (when asked) model output data are written
out_ana_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/results/eco_lai/lai/' # Folder where PData (when asked) analysis output data are written
graphics_dir = '/cnrm/vegeo/munier/e2o/GLO_1D/results/eco_lai/figures/' # Folder from where graphics files are written in the post-processing step.

# -------------------------------
# POST-PROCESSING
# -------------------------------
post_mod_from_pickle   = True # if True: try to read pickle files; if False or no pickle files: read model output files and save to pickle files.
post_ana_from_pickle   = True
post_obs_from_pickle   = True
extra_out_vars         = []#,'WG2','RUNOFFC','EVAPC','DRAINC','RC_ECO'] # GPPC
list_time_windows      = ['day' , 'month' , 'season' , 'year' , 'all']
var_spatial_maps_out   = [False , True    , False    , False  , False]
var_time_series_out    = [False , True    , True     , True   , False]
stats_spatial_maps_out = [False , True    , True     , True   , False]
stats_time_series_out  = [False , False   , False    , False  , False]
stats_table_out        = [False , False   , False    , False  , False]
extra_diagnostics      = {'patch'              :['LAI','SSM']}
patch_out              = [4,5,6,7,8,9,10,11,12] # patch number between 1 and 12
fig_dpi                = 300 # Resolution for saving raster figures.
# COPERNICUS figures to save to put in the final report
copernicus_figs        = False
#copernicus_fileForFigs = 'copernicus_figures.csv'

