
exp_name = 'sfx-trip' # Name of experiment
#exp_name = 'LAImin'

# -------------------------------
# ASSIMILATION
# -------------------------------
analysis_period = ['2000-01-01', '2000-12-31'] # Analysis period.
assim_hour = 9 # Assimilation hour (in range(24)).

# -------------------------------
# MODEL
# -------------------------------
# Grid
mod_grid  = {'type': 'uniform', 'res': 1. , 'zone': [[-180,90],[180,-90]]} # Model grid type ([[ul_lon,ul_lat],[lr_lon,lr_lat]])
trip_grid = {'type': 'uniform', 'res': 0.5, 'zone': [[-180,90],[180,-90]]}
# Output format
mod_out_format = {'format':'netcdf'} # Model output file format.

# -------------------------------
# FORCING
# -------------------------------
get_raw_forcing = False # Retrieve raw forcing files (True) or not (False)
freq_forcing = 3 # Time frequency of the forcing data in hours
nc_version = 'NETCDF3_64BIT' # NetCDF version; possbile values: 'NETCDF4', 'NETCDF4_CLASSIC', 'NETCDF3_64BIT', 'NETCDF3_CLASSIC'
raw_forcing_format = 'SFX_NC' # files already in SFX format, 1 file per month
FORC_VARS  = {'Tair':'Tair','Qair':'Qair','Wind':'Wind','DIR_SWdown':'DIR_SWdown','SCA_SWdown':'SCA_SWdown','LWdown':'LWdown','PSurf':'PSurf','Rainf':'Rainf','Snowf':'Snowf','CO2air':'CO2air','Wind_DIR':'Wind_DIR'}
#raw_forcing_format = 'VAR_NC' # seperated files for each variable (1 directory per variable), monthly files
#FORC_VARS  = {'Tair':'Tair' ,'Qair':'Qair' ,'Wind':'Wind' ,'DIR_SWdown':'SWdown' ,'LWdown':'LWdown' ,'Rainf':'Rainf'     ,'Snowf':'Snowf'     ,'PSurf':'PSurf'}

# -------------------------------
# OBSERVATIONS
# -------------------------------
# Variables to be assimilated (names and folders)
obs_names                   = ['E_ACTUAL']
obs_datasets                = ['GLEAM']
to_assim                    = [False]
obs_from_pickle             = [True]
# Grids
obs_grids                   = ['def']
grid_transform_methods      = ['def']
grid_transform_parameters   = ['def']
# Screening with land surface properties
surf_screen_features        = [[]]
surf_screen_thresholds      = [[]]
# Screening with model output variables
screen_mod_names            = [[]]
screen_mod_steps            = [[]]
screen_mod_thres            = [[]]
# Re-scaling using model output variables
rescale_methods             = [None          ] # Methods for re-scaling observations using model output (None means no re-scaling).
rescale_mod_names           = [None          ] # Model output variables used for re-scaling observations.
rescale_mod_steps           = [None          ] # Time steps (in range(mod_out_nsteps)) of model output used for re-scaling.
rescale_max_miss            = [None          ] # Maximum fraction of missing observations for re-scaling a grid-point time series; if not satisfied, observations are set to missing.
rescale_calibs              = [None          ] # Booleans indicating whether the re-scaling parameters should be calibrated.
rescale_calib_periods       = [[]            ] # Periods (start and end year, both included) over which the re-scaling parameters are calibrated.

# -------------------------------
# DIRECTORIES
# -------------------------------
# Input
mod_pgd_path = '/home/muniers/LDAS/GLO_1D/'+exp_name+'/pgd_prep/' # Must include PGD.nc, PGD_fractown.nc and PREP.nc
raw_forcing_dir = '/cnrm/vegeo/munier/e2o/GLO_025D/forcings/monthly/' # Folder from where the raw forcing data is read
obs_root_dir = '/cnrm/vegeo/LDAS/OBSERVATIONS/' # Root folder of all observation files
openloop_dir = '/cnrm/vegeo/munier/e2o/GLO_1D/results/'+exp_name+'/ol/' # Folder from where openloop output is read in pre- and post-processing steps
analysis_dir = '/cnrm/vegeo/munier/e2o/GLO_1D/results/'+exp_name+'/ekf/' # Folder from where analysis output is read in post-processing step
# Output
forcing_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/forcings/daily/' # Folder where NetCDF (as well as intermediate binary) forcing files are written.
out_obs_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/observations/sfx-trip/GLEAM/' # Folder where processed (CANARI files) observations are written
out_mod_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/results/'+exp_name+'/ol/' # Folder where PData (when asked) model output data are written
out_ana_dir  = '/cnrm/vegeo/munier/e2o/GLO_1D/results/'+exp_name+'/ekf/' # Folder where PData (when asked) analysis output data are written
graphics_dir = '/cnrm/vegeo/munier/e2o/GLO_1D/results/'+exp_name+'/figures/' # Folder from where graphics files are written in the post-processing step.

# -------------------------------
# POST-PROCESSING
# -------------------------------
post_mod_from_pickle   = True # if True: try to read pickle files; if False or no pickle files: read model output files and save to pickle files.
post_ana_from_pickle   = True
post_obs_from_pickle   = True
#post_zones             = {'def': None} # Default
#post_zones             = {'def': None, 'all': 'STN_Continents_1D.nc'}
post_zones             = {'all': 'STN_Continents_1D.nc'}
extra_out_vars         = ['EVAPC','DRAINC','RUNOFFC'] # GPPC
trip_vars              = ['QDIS']
list_time_windows      = ['day' , 'month' , 'season' , 'year' , 'all']
var_spatial_maps_out   = [False , False   , True     , False  , True ]
var_time_series_out    = [True  , True    , True     , True   , False]
stats_spatial_maps_out = [False , False   , True     , False  , True ]
stats_time_series_out  = [False , False   , True     , True   , False]
stats_table_out        = [False , False   , False    , False  , False]
extra_diagnostics      = {'increments_assim'   :['ANAL_INCR1'    , 'ANAL_INCR2'    ],\
                          'increments_names'   :['LAI increments', 'WG2 increments'],\
                          'jacobian_assim'     :['HO_2_2'   ,'HO_2_3'   ],\
                          'jacobian_names'     :['dLAI/dWG2','dLAI/dWG3'],\
                          'jacobian_limits'    :[[-1e-3,1e-3],[-1e-1,1e-1],[-3e-1,3e-1],[-3e-1,3e-1]],\
                         }
patch_out              = [4,5,6,7,8,9,10,11,12] # patch number between 1 and 12
modexp_names           = 'def' # if 'def': ['Model','Analysis']
fig_dpi                = 300 # Resolution for saving raster figures.
# COPERNICUS figures to save to put in the final report
copernicus_figs        = True
copernicus_fileForFigs = 'copernicus_figures.csv'


# -------------------------------
# For tests
# -------------------------------
if True:
  list_time_windows      = ['all']
  var_spatial_maps_out   = [True]
  var_time_series_out    = [False]
  stats_spatial_maps_out = [False]
  stats_time_series_out  = [False]
  stats_table_out        = [False]
