import pylab as pl
import myPyFunc as my
from matplotlib import pylab as plt
from tqdm import trange
from time import time
import pdb

# Parameters
zone = [[-10,30],[40,0]]

obs_res = 1./4

radius = 1

updown = 'down'

#mod_lon,mod_lat = [0.47,15.6,40-obs_res/2-1e-6],[10.5,5.8,-0.5]
mod_lon,mod_lat = pl.meshgrid(pl.arange(-179.5,180),pl.arange(-89.5,90))
mod_lon,mod_lat = mod_lon.ravel(),mod_lat.ravel()


# Code

def _res2lonlat(res,zone,pix_center=0,cut_buffer=0):
    lon = pl.arange(zone[0][0]-cut_buffer+(1-pix_center)*res/2,zone[1][0]+cut_buffer,res)
    lat = pl.arange(zone[1][1]-cut_buffer+(1-pix_center)*res/2,zone[0][1]+cut_buffer,res)
    return lon,lat

def lookupUpscale(mod_lon,mod_lat,obs_lon,obs_lat,assim_dist):
    '''
    Lookup table for upscale resampling for uniform observation grids and list of model points
    '''
    missing_value = -999
    Kobs = pl.arange(len(obs_lon)*len(obs_lat),dtype='uint32').reshape(len(obs_lat),len(obs_lon))
    obs_res = float(obs_lon[1]-obs_lon[0])
    b = assim_dist/obs_res
    c_lon = (mod_lon-obs_lon[0])/obs_res
    c_lat = (mod_lat-obs_lat[0])/obs_res
    n_lon,n_lat = len(obs_lon),len(obs_lat)
    Npnt = int((2*b+1)**2)
    LUT = missing_value*pl.ones((len(mod_lon),Npnt),dtype='int32')
    for k in range(len(mod_lon)):
      I = slice(max(min(int(pl.ceil(c_lon[k]-b)),n_lon),0),\
                max(min(int(pl.floor(c_lon[k]+b))+1,n_lon),0))
      J = slice(max(min(int(pl.ceil(c_lat[k]-b)),n_lat),0),\
                max(min(int(pl.floor(c_lat[k]+b))+1,n_lat),0))
      tmp = Kobs[J,I].ravel()
      LUT[k,:len(tmp)] = tmp
    Npnt = pl.argmax(pl.all(LUT==missing_value,axis=0))
    LUT = LUT[:,:Npnt]
    return LUT,Npnt,missing_value

def lookupDownscale(mod_lon,mod_lat,obs_lon,obs_lat,assim_dist):
    '''
    '''
    missing_value = -999
    n_lon,n_lat = len(obs_lon),len(obs_lat)
    obs_res = float(obs_lon[1]-obs_lon[0])
    Ilon = pl.int_(pl.round_((mod_lon-obs_lon[0])/obs_res))
    Ilat = pl.int_(pl.round_((mod_lat-obs_lat[0])/obs_res))
    r2 = assim_dist**2
    N = int(pl.ceil(assim_dist/obs_res))
    Npnt = (2*N+1)**2
    LUT = missing_value*pl.ones((len(mod_lon),Npnt),dtype='int32')
    for k in trange(len(mod_lon)):
      for j in range(-N,N+1):
        for i in range(-N,N+1):
          if Ilon[k]+i>=0 and Ilon[k]+i<n_lon and Ilat[k]+j>=0 and Ilat[k]+j<n_lat and \
            (mod_lon[k]-obs_lon[Ilon[k]+i])**2+(mod_lat[k]-obs_lat[Ilat[k]+j])**2<=r2:
            LUT[k,i+N+(j+N)*(2*N+1)] = Ilon[k]+i+(Ilat[k]+j)*n_lon
    LUT[:,pl.any(LUT!=missing_value,axis=0)]
    Npnt = LUT.shape[1]
    return LUT,Npnt,missing_value


obs_zone = pl.array(zone)
obs_lon,obs_lat = _res2lonlat(obs_res,obs_zone)

t0 = time()
if updown=='up': LUT,Npnt,_ = lookupUpscale(mod_lon,mod_lat,obs_lon,obs_lat,radius)
else           : LUT,Npnt,_ = lookupDownscale(mod_lon,mod_lat,obs_lon,obs_lat,radius)
t0 = time()-t0
print('Elapsed time: {:0.4f}'.format(t0))


# Test figure
X,Y = pl.meshgrid(obs_lon,obs_lat)
X = X.ravel()
Y = Y.ravel()

#k = 0
k = pl.argmax(pl.logical_and(mod_lon>16,mod_lat>0))
lut = LUT[k,:]
lut = lut[lut!=-999]
plt.figure(2)
plt.clf()
plt.plot(mod_lon[k],mod_lat[k],'bo')
plt.plot(X[lut],Y[lut],'rx')
r = [mod_lon[k]-radius,mod_lon[k]+radius,mod_lat[k]-radius,mod_lat[k]+radius]
z = [zone[0][0],zone[1][0],zone[1][1],zone[0][1]]
o = [obs_zone[0][0],obs_zone[1][0],obs_zone[1][1],obs_zone[0][1]]
if updown=='up': plt.plot([r[0],r[1],r[1],r[0],r[0]],[r[2],r[2],r[3],r[3],r[2]],'g')
else           : plt.gca().add_artist(plt.Circle([mod_lon[k],mod_lat[k]],radius,ec='g'))
plt.plot([z[0],z[1],z[1],z[0],z[0]],[z[2],z[2],z[3],z[3],z[2]],'y--')
plt.plot([o[0],o[1],o[1],o[0],o[0]],[o[2],o[2],o[3],o[3],o[2]],'y')
plt.axis('equal')
#plt.axis((mod_lon[i]-radius,mod_lon[i]+radius,mod_lat[j]-radius,mod_lat[j]+radius))

