import pandas as pd
import numpy as np
from matplotlib import pyplot as plt
import myPyFunc as my
import sys

zone = [[30,60],[90,25]]
zone = [[40,30],[60,25]]
#zone = [[-12,70],[40,25]]
zone = [[-9.5,41.5],[-7.5,39.5]]

mod_grid = {'res': 1/20., 'type': 'uniform', 'zone': zone}

dataset = 'BIOPAR_12D'
obs_var = 'LAI'

def _res2lonlat(res,zone,pix_center=0,cut_buffer=0):
    lon = np.arange(zone[0][0]-cut_buffer+(1-pix_center)*res/2,zone[1][0]+cut_buffer,res)
    lat = np.arange(zone[1][1]-cut_buffer+(1-pix_center)*res/2,zone[0][1]+cut_buffer,res)
    return lon,lat



# get regions available for variable
regions_file = '/cnrm/vegeo/LDAS/OBSERVATIONS/{0}/{1}/{0}_{1}_REGIONS.csv'.format(dataset,obs_var)
regions = pd.read_csv(regions_file,sep=',',header=0,index_col=0).T
# compute buffer around model grid
cut_buffer = 2*mod_grid['res']
# lon/lat of model grid
z = mod_grid['zone']
zbuf = np.array(z)+cut_buffer*np.array([[-1,1],[1,-1]])
mod_res = mod_grid['res']
mod_lon,mod_lat = _res2lonlat(mod_res,z)
# slice observation grid for each region
obs_slices = dict()
r = regions[regions.keys()[0]]
obs_res = (r['lr_lon']-r['ul_lon'])/(r['nlon']-r['pix_center'])
eps = obs_res*1e-4
obs_lon_min = r['ul_lon']+obs_res/2*(1-r['pix_center'])+obs_res*np.ceil((zbuf[0][0]-r['ul_lon'])/obs_res-(1-r['pix_center'])/2)
obs_lat_min = r['lr_lat']+obs_res/2*(1-r['pix_center'])+obs_res*np.ceil((zbuf[1][1]-r['lr_lat'])/obs_res-(1-r['pix_center'])/2)
obs_lon,obs_lat = _res2lonlat(obs_res,[[obs_lon_min,zbuf[0][1]],[zbuf[1][0],obs_lat_min]],pix_center=1)
for r_name,r in regions.iteritems():
    zobs = np.array([[r['ul_lon'],r['ul_lat']],[r['lr_lon'],r['lr_lat']]])
    # does this region intersect the model zone?
    to_read = (zobs[0][1]>=zbuf[1][1] and zobs[0][0]<=zbuf[1][0] and zobs[1][1]<=zbuf[0][1] and zobs[1][0]>=zbuf[0][0])
    # observation slices for this region
    lon,lat = _res2lonlat(obs_res,zobs,pix_center=r['pix_center'])
    I = (lat>=zbuf[1][1]-eps)*(lat<=zbuf[0][1]+eps)*(lat>=obs_lat[0]-eps)*(lat<=obs_lat[-1]+eps)
    row_slice_in  = slice(I.argmax(),len(I)-I[::-1].argmax())
    I = (lon>=zbuf[0][0]-eps)*(lon<=zbuf[1][0]+eps)*(lon>=obs_lon[0]-eps)*(lon<=obs_lon[-1]+eps)
    col_slice_in  = slice(I.argmax(),len(I)-I[::-1].argmax())
    I = (obs_lat>=zobs[1][1]-eps)*(obs_lat<=zobs[0][1]+eps)*(obs_lat>=lat[0]-eps)*(obs_lat<=lat[-1]+eps)
    row_slice_out  = slice(I.argmax(),len(I)-I[::-1].argmax())
    I = (obs_lon>=zobs[0][0]-eps)*(obs_lon<=zobs[1][0]+eps)*(obs_lon>=lon[0]-eps)*(obs_lon<=lon[-1]+eps)
    col_slice_out  = slice(I.argmax(),len(I)-I[::-1].argmax())
    obs_slices[r_name] = {'to_read': to_read, \
      'row_slice_in': row_slice_in, 'col_slice_in': col_slice_in, 'row_slice_out': row_slice_out, 'col_slice_out': col_slice_out}
    #sys.exit()


for r,s in obs_slices.iteritems():
    if not s['to_read']: continue
    if s['row_slice_in'].stop-s['row_slice_in'].start != s['row_slice_out'].stop-s['row_slice_out'].start:
        print 'error in row_slice for '+r
    if s['col_slice_in'].stop-s['col_slice_in'].start != s['col_slice_out'].stop-s['col_slice_out'].start:
        print 'error in col_slice for '+r

sys.exit()


mod_lon_mesh,mod_lat_mesh = np.meshgrid(mod_lon,mod_lat)
obs_lon_mesh,obs_lat_mesh = np.meshgrid(obs_lon,obs_lat)

r = regions['AFRI']
S = obs_slices['AFRI']
zobs = [[r['ul_lon'],r['ul_lat']],[r['lr_lon'],r['lr_lat']]]
x,y = _res2lonlat(obs_res,zobs,pix_center=r['pix_center'])
x_mesh,y_mesh = np.meshgrid(x,y)

plt.figure(1); plt.clf()
my.pmap()
plt.plot([z[0][0],z[1][0],z[1][0],z[0][0],z[0][0]],[z[0][1],z[0][1],z[1][1],z[1][1],z[0][1]],'b--')
plt.plot([zbuf[0][0],zbuf[1][0],zbuf[1][0],zbuf[0][0],zbuf[0][0]],[zbuf[0][1],zbuf[0][1],zbuf[1][1],zbuf[1][1],zbuf[0][1]],'b')
plt.plot([zobs[0][0],zobs[1][0],zobs[1][0],zobs[0][0],zobs[0][0]],[zobs[0][1],zobs[0][1],zobs[1][1],zobs[1][1],zobs[0][1]],'y')
if S['to_read']:
    plt.plot(x_mesh[S['row_slice_in'],S['col_slice_in']].ravel(),y_mesh[S['row_slice_in'],S['col_slice_in']].ravel(),'go')
    plt.plot(obs_lon_mesh[S['row_slice_out'],S['col_slice_out']].ravel(),obs_lat_mesh[S['row_slice_out'],S['col_slice_out']].ravel(),'r+')



