#!/bin/python
import os,sys
import pandas as pd
import pylab as pl
from re import match
from ldasModule import getObsRegions


obs_root_dir = '/cnrm/vegeo/LDAS/OBSERVATIONS/' # Root folder of all observation files
obs_dataset = 'GOME2'
obs_var = 'SIF'


# 1. Get available dates and file names for the first region
# ----------------------------------------------------------------------------------
obs_daily = True
regions = getObsRegions(obs_root_dir,obs_dataset,obs_var)
obs_region = regions.keys()[0]
obs_data_dir = os.path.normpath('/'.join((obs_root_dir,obs_dataset,obs_var,obs_region)))
files_name = []
# BIOPAR DATASET
if obs_dataset == 'BIOPAR':
    if obs_var in ['LAI','SWI','FAPAR','SA','FCOVER']:
        assim_dates = pl.array(sorted([pd.to_datetime(d) for d in os.listdir(obs_data_dir) if match('\d{8}',d) is not None]))
        for curr_date in assim_dates:
            date_str = curr_date.strftime('%Y%m%d')
            tmp = [f for f in os.listdir(obs_data_dir+'/'+date_str) if match('g2_BIOPAR_.*h5$',f) is not None][0]
            files_name.append(os.path.join(date_str,tmp).replace(obs_region,'XXXREGIONXXX'))
    elif obs_var == 'LST':
        obs_daily = False
        r_e = 'g2_BIOPAR_LST_(\d{10})00.+'
        assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d%H') for g in os.walk(obs_data_dir) for f in g[2]]))
        for curr_date in assim_dates:
            date_str = curr_date.strftime('%Y%m%d%H')
            tmp_dir = '{1}/{2}/{3}'.format(obs_data_dir,date_str[:4],date_str[4:6],date_str[6:8])
            tmp = [f for f in os.listdir(obs_data_dir+'/'+tmp_dir) if match('g2_BIOPAR_.*_{0}00_.*h5$'.format(date_str),f) is not None][0]
            files_name.append(os.path.join(tmp_dir,tmp).replace(obs_region,'XXXREGIONXXX'))
# BIOPAR_12D DATASET
elif obs_dataset == 'BIOPAR_12D':
    if obs_var == 'LAI':
        #obs_data_dir = obs_data_dir.replace('GLOBE','EURO')
        r_e = 'g2_BIOPAR_LAI_(\d{8})0000.+'
        assim_dates = pl.sort(pl.array([pd.datetime.strptime(match(r_e,f).group(1),'%Y%m%d') for f in os.listdir(obs_data_dir)]))
        for curr_date in assim_dates:
            date_str = curr_date.strftime('%Y%m%d%H')
            tmp = [f for f in os.listdir(obs_data_dir) if match('g2_BIOPAR_.*_{0}00_.*h5'.format(date_str),f) is not None][0]
            files_name.append(tmp.replace(obs_region,'XXXREGIONXXX'))
# ESACCI DATASET
elif obs_dataset == 'ESACCI':
    if obs_var == 'SSM':
        r_e = 'ESACCI-SOILMOISTURE-L3S-SSMV-COMBINED-(\d{8})000000-fv02.2.nc'
        date_str = []
        for d in os.listdir(obs_data_dir):
            if not os.path.isdir(obs_data_dir+'/'+d): continue
            date_str = date_str+[match(r_e,f).groups(1)[0] for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None]
        assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
        for curr_date in assim_dates:
            date_str = curr_date.strftime('%Y%m%d%H')
            files_name.append('{0}/ESACCI-SOILMOISTURE-L3S-SSMV-COMBINED-{1}0000-fv02.2.nc'.format(date_str[:4],date_str))
# GOME2 DATASET
elif obs_dataset == 'GOME2':
    if obs_var == 'SIF':
        r_e = 'GOME-2_SIF_(\d{8}).nc'
        date_str = []
        for d in os.listdir(obs_data_dir):
            if not os.path.isdir(obs_data_dir+'/'+d): continue
            date_str = date_str+[match(r_e,f).groups(1)[0] for f in os.listdir(obs_data_dir+'/'+d) if match(r_e,f) is not None]
        assim_dates = pl.array(sorted([pd.to_datetime(d) for d in date_str]))
        for curr_date in assim_dates:
            date_str = curr_date.strftime('%Y%m%d')
            files_name.append('{0}/GOME-2_SIF_{1}.nc'.format(date_str[:4],date_str))

# 2. Check if dates are the same for the other regions
# ----------------------------------------------------------------------------------
I_remove = []
for i,f in zip(range(len(files_name)),files_name): #d,f in zip(assim_dates,files_name):
    for r in [r for r in regions if not r==obs_region]:
        obs_data_dir = os.path.normpath('/'.join((obs_root_dir,obs_dataset,obs_var,r)))
        if not os.path.isfile(os.path.join(obs_data_dir,f).replace('XXXREGIONXXX',r)):
            print('Date {0} is available for {1} but not for {2}. It has been removed'\
                .format(assim_dates[i].strftime('%Y%m%d%H'),obs_region,r))
            print(os.path.join(obs_data_dir,f).replace('XXXREGIONXXX',r))
            I_remove.append(i)
            break
assim_dates = pl.delete(assim_dates,I_remove)
files_name = pl.delete(files_name,I_remove)


# 3. Write the date file
# ----------------------------------------------------------------------------------
var_date_file = os.path.join(obs_root_dir,obs_dataset,obs_var,obs_dataset+'_'+obs_var+'_DATES.csv')
with open(var_date_file,'w') as fid:
    for d,f in zip(assim_dates,files_name):
        dat = d.strftime('%Y%m%d%H')
        if obs_daily: dat = dat[:-2]+'XX'
        fid.write('{0},{1}\n'.format(dat,f))



